@@ -3,7 +3,7 @@
 //
 // THIS MODULE IS PUBLICLY LICENSED
 //
-// Copyright 2001-2011 by Wilson Snyder.  This program is free software;
+// Copyright 2001-2012 by Wilson Snyder.  This program is free software;
 // you can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 //
@@ -86,8 +86,8 @@ private:
     NameMap*			m_namemapp;	///< List of names for the header
     static vector<VerilatedVcd*>	s_vcdVecp;	///< List of all created traces
 
-    inline size_t bufferSize() { return 256*1024; }  // See below for slack calculation
-    inline size_t bufferInsertSize() { return 16*1024; }
+    inline static size_t bufferSize() { return 256*1024; }  // See below for slack calculation
+    inline static size_t bufferInsertSize() { return 16*1024; }
     void bufferFlush();
     void bufferCheck() {
 	// Flush the write buffer if there's not enough space left for new information
@@ -110,6 +110,7 @@ private:
     void dumpHeader();
     void dumpPrep (vluint64_t timeui);
     void dumpFull (vluint64_t timeui);
+    // cppcheck-suppress functionConst
     void dumpDone ();
     inline void printCode (vluint32_t code) {
 	if (code>=(94*94*94)) *m_writep++ = ((char)((code/94/94/94)%94+33));
@@ -117,7 +118,7 @@ private:
 	if (code>=(94))       *m_writep++ = ((char)((code/94)%94+33));
 	*m_writep++ = ((char)((code)%94+33));
     }
-    string stringCode (vluint32_t code) {
+    static string stringCode (vluint32_t code) {
 	string out;
 	if (code>=(94*94*94)) out += ((char)((code/94/94/94)%94+33));
 	if (code>=(94*94))    out += ((char)((code/94/94)%94+33));
@@ -141,6 +142,8 @@ public:
 	m_evcd = false;
 	m_scopeEscape = '.';  // Backward compatibility
 	m_wroteBytes = 0;
+	m_fd = 0;
+	m_fullDump = true;
     }
     ~VerilatedVcd();
 
