@@ -8,7 +8,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -26,8 +26,10 @@
 #include "verilatedos.h"
 #include "V3Error.h"
 #include "V3Number.h"
+#include "V3Global.h"
 #include <vector>
 #include <cmath>
+#include <map>
 
 #include "V3Ast__gen_classes.h"	// From ./astgen
 // Things like:
@@ -35,6 +37,7 @@
 
 // Hint class so we can choose constructors
 class AstLogicPacked {};
+class AstBitPacked {};
 
 //######################################################################
 
@@ -288,11 +291,13 @@ public:
 	case INTEGER:	return 32;
 	case LOGIC:	return 1;
 	case LONGINT:	return 64;
-	case DOUBLE:	return 64;
-	case FLOAT:	return 32;
+	case DOUBLE:	return 64;  // opaque
+	case FLOAT:	return 32;  // opaque
 	case SHORTINT:	return 16;
 	case TIME:	return 64;
-	case STRING:	return 64;  // Just the pointer, for today
+	case STRING:	return 64;  // opaque  // Just the pointer, for today
+	case SCOPEPTR:	return 0;   // opaque
+	case CHARPTR:	return 0;   // opaque
 	default: return 0;
 	}
     }
@@ -309,7 +314,7 @@ public:
     bool isSloppy() const { // Don't be as anal about width warnings
 	return !(m_e==LOGIC || m_e==BIT);
     }
-    bool isBitLogic() const { // Don't be as anal about width warnings
+    bool isBitLogic() const { // Bit/logic vector types; can form a packed array
 	return (m_e==LOGIC || m_e==BIT);
     }
     bool isDpiUnsupported() const {
@@ -330,7 +335,7 @@ public:
 
 enum AstSignedState {
     // This can't be in the fancy class as the lexer union will get upset
-    signedst_NOP=0, signedst_SIGNED=1, signedst_UNSIGNED=2
+    signedst_NOSIGNED=0, signedst_UNSIGNED=1, signedst_SIGNED=2
 };
 
 //######################################################################
@@ -859,16 +864,17 @@ public:
     void	numericFrom(AstNode* fromp) { numeric(fromp->numeric()); }
     void	numeric(AstNumeric flag) { m_numeric = (int)flag; if (flag.isDouble()) width(64,64); }
     AstNumeric	numeric() const { return AstNumeric(m_numeric); }
-    bool	isDouble() const { return numeric().isDouble(); }
-    bool	isSigned() const { return numeric().isSigned(); }
     void	isSigned(bool flag) { numeric(flag ? AstNumeric::SIGNED : AstNumeric::UNSIGNED); }
     bool	isUnsigned() const { return numeric().isUnsigned(); }
     void	didWidth(bool flag) { m_didWidth=flag; }
     bool	didWidth() const { return m_didWidth; }
+    bool	didWidthAndSet() { if (didWidth()) return true; didWidth(true); return false;}
     void	doingWidth(bool flag) { m_doingWidth=flag; }
     bool	doingWidth() const { return m_doingWidth; }
     bool	isQuad() const { return (width()>VL_WORDSIZE && width()<=VL_QUADSIZE); }
     bool	isWide() const { return (width()>VL_QUADSIZE); }
+    bool	isDouble() const;
+    bool	isSigned() const;
 
     AstNUser*	user1p() const {
 	// Slows things down measurably, so disabled by default
@@ -931,15 +937,26 @@ public:
     bool	isAllOnes();
     bool	isAllOnesV();  // Verilog width rules apply
 
-    // METHODS
+    // METHODS - data type changes especially for initial creation
+    void	dtypeChgLogicBool()	{ numeric(AstNumeric::UNSIGNED); width(1,1); }
+    void	dtypeChgDouble()	{ numeric(AstNumeric::DOUBLE); }
+    void	dtypeChgSigned32()	{ numeric(AstNumeric::SIGNED); width(VL_WORDSIZE,VL_WORDSIZE); }
+    void	dtypeChgUInt32()	{ numeric(AstNumeric::UNSIGNED); width(VL_WORDSIZE,VL_WORDSIZE); }
+    void	dtypeChgUInt64()	{ numeric(AstNumeric::UNSIGNED); width(VL_QUADSIZE,VL_QUADSIZE); }
+
+    // METHODS - dump and error
+    void	v3errorEnd(ostringstream& str) const;
+    virtual void dump(ostream& str=cout);
+    void	dumpGdb(); // For GDB only
+    void	dumpGdbHeader() const;
+
+    // METHODS - Tree modifications
     AstNode*	addNext(AstNode* newp);		// Returns this, adds to end of list
     AstNode*	addNextNull(AstNode* newp);	// Returns this, adds to end of list, NULL is OK
     void	addNextHere(AstNode* newp);	// Adds after speced node
     void	addPrev(AstNode* newp) { replaceWith(newp); newp->addNext(this); }
     void	addHereThisAsNext(AstNode* newp); // Adds at old place of this, this becomes next
     void	replaceWith(AstNode* newp);	// Replace current node in tree with new node
-    void	v3errorEnd(ostringstream& str) const;
-    virtual void dump(ostream& str=cout);
     AstNode*	unlinkFrBack(AstNRelinker* linkerp=NULL);  // Unlink this from whoever points to it.
     AstNode*	unlinkFrBackWithNext(AstNRelinker* linkerp=NULL);  // Unlink this from whoever points to it, keep entire next list with unlinked node
     void	swapWith(AstNode* bp);
@@ -956,11 +973,13 @@ public:
     void	checkTree();  // User Interface version
     void	dumpPtrs(ostream& str=cout) const;
     void	dumpTree(ostream& str=cout, const string& indent="    ", int maxDepth=0);
+    void	dumpTreeGdb(); // For GDB only
     void	dumpTreeAndNext(ostream& str=cout, const string& indent="    ", int maxDepth=0);
     void	dumpTreeFile(const string& filename, bool append=false);
 
     // METHODS - queries
-    virtual bool isSplittable() const { return true; }	// Else a $display, etc, that must be ordered with other displays
+    virtual bool isPure() const { return true; }	// Else a $display, etc, that must be ordered with other displays
+    virtual bool isBrancher() const { return false; }	// Changes control flow, disable some optimizations
     virtual bool isGateOptimizable() const { return true; }	// Else a AstTime etc that can't be pushed out
     virtual bool isSubstOptimizable() const { return true; }	// Else a AstTime etc that can't be substituted out
     virtual bool isPredictOptimizable() const { return true; }	// Else a AstTime etc which output can't be predicted from input
@@ -1129,7 +1148,7 @@ struct AstNodeCond : public AstNodeTriop {
 };
 
 struct AstNodePreSel : public AstNode {
-    // Something that becomes a AstSel
+    // Something that becomes an AstSel
     AstNodePreSel(FileLine* fl, AstNode* lhs, AstNode* rhs, AstNode* ths)
 	: AstNode(fl) {
 	setOp1p(lhs); setOp2p(rhs); setNOp3p(ths); }
@@ -1253,16 +1272,18 @@ private:
     string	m_name;		// Name of variable
     string	m_hiername;	// Scope converted into name-> for emitting
     bool	m_hierThis;	// Hiername points to "this" function
+    void init();
 public:
     AstNodeVarRef(FileLine* fl, const string& name, bool lvalue)
 	: AstNodeMath(fl), m_lvalue(lvalue), m_varp(NULL), m_varScopep(NULL),
 	  m_packagep(NULL), m_name(name), m_hierThis(false) {
+	init();
     }
     AstNodeVarRef(FileLine* fl, const string& name, AstVar* varp, bool lvalue)
 	: AstNodeMath(fl), m_lvalue(lvalue), m_varp(varp), m_varScopep(NULL),
 	  m_packagep(NULL), m_name(name), m_hierThis(false) {
 	// May have varp==NULL
-	if (m_varp) widthSignedFrom((AstNode*)m_varp);
+	init();
     }
     ASTNODE_BASE_FUNCS(NodeVarRef)
     virtual bool broken() const;
@@ -1310,10 +1331,15 @@ struct AstNodeDType : public AstNode {
     AstNodeDType(FileLine* fl) : AstNode(fl) {}
     ASTNODE_BASE_FUNCS(NodeDType)
     // Accessors
+    virtual void dump(ostream& str);
     virtual AstBasicDType* basicp() const = 0;  // (Slow) recurse down to find basic data type
     virtual AstNodeDType* skipRefp() const = 0;  // recurses over typedefs to next non-typeref type
     virtual int widthAlignBytes() const = 0; // (Slow) recurses - Structure alignment 1,2,4 or 8 bytes (arrays affect this)
     virtual int widthTotalBytes() const = 0; // (Slow) recurses - Width in bytes rounding up 1,2,4,8,12,...
+    virtual bool maybePointedTo() const { return true; }
+    AstNodeDType* dtypeDimensionp(int depth);
+    pair<uint32_t,uint32_t> dimensions();
+    uint32_t	arrayElements();	// 1, or total multiplication of all dimensions
 };
 
 struct AstNodeSel : public AstNodeBiop {
@@ -1489,10 +1515,15 @@ public:
 //######################################################################
 // Inline ACCESSORS
 
+inline bool AstNode::isDouble() const { return numeric().isDouble(); }
+inline bool AstNode::isSigned() const { return numeric().isSigned(); }
+
 inline bool AstNode::isZero()     { return (this->castConst() && this->castConst()->num().isEqZero()); }
 inline bool AstNode::isNeqZero()  { return (this->castConst() && this->castConst()->num().isNeqZero()); }
 inline bool AstNode::isOne()      { return (this->castConst() && this->castConst()->num().isEqOne()); }
-inline bool AstNode::isAllOnes()  { return (this->castConst() && this->castConst()->num().isEqAllOnes(this->width())); }
-inline bool AstNode::isAllOnesV() { return (this->castConst() && this->castConst()->num().isEqAllOnes(this->widthMin())); }
+inline bool AstNode::isAllOnes()  { return (this->castConst() && this->castConst()->isEqAllOnes()); }
+inline bool AstNode::isAllOnesV() { return (this->castConst() && this->castConst()->isEqAllOnesV()); }
+
+inline void AstNodeVarRef::init() { if (m_varp) widthSignedFrom((AstNode*)m_varp); }
 
 #endif // Guard
