@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -245,7 +245,7 @@ string AstVar::scType() const {
     }
 }
 
-AstNodeDType* AstVar::dtypeDimensionp(int dimension) const {
+AstNodeDType* AstNodeDType::dtypeDimensionp(int dimension) {
     // dimension passed from AstArraySel::dimension
     // Dimension 0 means the VAR itself, 1 is the closest SEL to the AstVar,
     // which is the lowest in the dtype list.
@@ -258,7 +258,7 @@ AstNodeDType* AstVar::dtypeDimensionp(int dimension) const {
     //	      *or*	VAR a (ARRAYSEL0 (ARRAYSEL1 (ARRAYSEL2 (ARRAYSEL3 (DT))))
     //	   SEL1 needs to select from entire variable which is a pointer to ARRAYSEL0
     int dim = 0;
-    for (AstNodeDType* dtypep=this->dtypep(); dtypep; ) {
+    for (AstNodeDType* dtypep=this; dtypep; ) {
 	dtypep = dtypep->skipRefp();  // Skip AstRefDType/AstTypedef, or return same node
 	if (AstArrayDType* adtypep = dtypep->castArrayDType()) {
 	    if ((dim++)==dimension) {
@@ -269,7 +269,7 @@ AstNodeDType* AstVar::dtypeDimensionp(int dimension) const {
 	}
 	else if (AstBasicDType* adtypep = dtypep->castBasicDType()) {
 	    // AstBasicDType - nothing below, return null
-	    if (adtypep->rangep()) {
+	    if (adtypep->isRanged()) {
 		if ((dim++) == dimension) {
 		    return adtypep;
 		}
@@ -282,9 +282,9 @@ AstNodeDType* AstVar::dtypeDimensionp(int dimension) const {
     return NULL;
 }
 
-uint32_t AstVar::arrayElements() const {
+uint32_t AstNodeDType::arrayElements() {
     uint32_t entries=1;
-    for (AstNodeDType* dtypep=this->dtypep(); dtypep; ) {
+    for (AstNodeDType* dtypep=this; dtypep; ) {
 	dtypep = dtypep->skipRefp();  // Skip AstRefDType/AstTypedef, or return same node
 	if (AstArrayDType* adtypep = dtypep->castArrayDType()) {
 	    entries *= adtypep->elementsConst();
@@ -298,11 +298,11 @@ uint32_t AstVar::arrayElements() const {
     return entries;
 }
 
-pair<uint32_t,uint32_t> AstVar::dimensions() const {
+pair<uint32_t,uint32_t> AstNodeDType::dimensions() {
     // How many array dimensions (packed,unpacked) does this Var have?
     uint32_t packed = 0;
     uint32_t unpacked = 0;
-    for (AstNodeDType* dtypep=this->dtypep(); dtypep; ) {
+    for (AstNodeDType* dtypep=this; dtypep; ) {
 	dtypep = dtypep->skipRefp();  // Skip AstRefDType/AstTypedef, or return same node
 	if (AstArrayDType* adtypep = dtypep->castArrayDType()) {
 	    if (adtypep->isPacked()) packed += 1;
@@ -496,21 +496,21 @@ void AstWhile::addNextStmt(AstNode* newp, AstNode* belowp) {
 //======================================================================
 // Per-type Debugging
 
-void AstNode::dump(ostream& os) {
-    os<<typeName()<<" "<<(void*)this
+void AstNode::dump(ostream& str) {
+    str<<typeName()<<" "<<(void*)this
 	//<<" "<<(void*)this->m_backp
-      <<" <e"<<dec<<editCount()
-      <<((editCount()>=editCountLast())?"#>":">")
-      <<" {"<<fileline()->filenameLetters()<<dec<<fileline()->lineno()<<"}"
-      <<" "<<(isSigned()?"s":"")
-      <<(isDouble()?"d":"")
-      <<"w"<<(widthSized()?"":"u")<<width();
-    if (!widthSized()) os<<"/"<<widthMin();
-    if (name()!="") os<<"  "<<AstNode::quoteName(name());
+       <<" <e"<<dec<<editCount()
+       <<((editCount()>=editCountLast())?"#>":">")
+       <<" {"<<fileline()->filenameLetters()<<dec<<fileline()->lineno()<<"}"
+       <<" "<<(isSigned()?"s":"")
+       <<(isDouble()?"d":"")
+       <<"w"<<(widthSized()?"":"u")<<width();
+    if (!widthSized()) str<<"/"<<widthMin();
+    if (name()!="") str<<"  "<<AstNode::quoteName(name());
 }
 
 void AstArrayDType::dump(ostream& str) {
-    this->AstNode::dump(str);
+    this->AstNodeDType::dump(str);
     if (isPacked()) str<<" [PACKED]";
 }
 void AstArraySel::dump(ostream& str) {
@@ -522,8 +522,9 @@ void AstAttrOf::dump(ostream& str) {
     str<<" ["<<attrType().ascii()<<"]";
 }
 void AstBasicDType::dump(ostream& str) {
-    this->AstNode::dump(str);
-    str<<" ["<<keyword().ascii()<<"]";
+    this->AstNodeDType::dump(str);
+    str<<" kwd="<<keyword().ascii();
+    if (isRanged() && !rangep()) str<<" range=["<<msb()<<":"<<lsb()<<"]";
     if (implicit()) str<<" [IMPLICIT]";
 }
 void AstCCast::dump(ostream& str) {
@@ -566,7 +567,7 @@ void AstRange::dump(ostream& str) {
     if (littleEndian()) str<<" [LITTLE]";
 }
 void AstRefDType::dump(ostream& str) {
-    this->AstNode::dump(str);
+    this->AstNodeDType::dump(str);
     if (defp()) { str<<" -> "; defp()->dump(str); }
     else { str<<" -> UNLINKED"; }
 }
@@ -580,6 +581,9 @@ void AstVarXRef::dump(ostream& str) {
     else if (varp()) { varp()->dump(str); }
     else { str<<"UNLINKED"; }
 }
+void AstNodeDType::dump(ostream& str) {
+    this->AstNode::dump(str);
+}
 void AstNodeModule::dump(ostream& str) {
     this->AstNode::dump(str);
     str<<"  L"<<level();
@@ -660,6 +664,7 @@ void AstNodeFTask::dump(ostream& str) {
 void AstBegin::dump(ostream& str) {
     this->AstNode::dump(str);
     if (unnamed()) str<<" [UNNAMED]";
+    if (hidden()) str<<" [HIDDEN]";
 }
 void AstCoverDecl::dump(ostream& str) {
     this->AstNode::dump(str);
