@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -204,14 +204,16 @@ private:
     virtual void visit(AstNode* nodep, AstNUser*) {
 	BrokenTable::setUnder(nodep,true);
 	if (nodep->broken()) {
-	    nodep->v3fatalSrc("Broken link in node (or something without maybePointedTo)\n");
+	    nodep->v3fatalSrc("Broken link in node (or something without maybePointedTo)");
 	}
-	if (v3Global.assertWidthsSame()) {
-	    if (nodep->width() != nodep->widthMin()) {
-		nodep->v3fatalSrc("Width != WidthMin\n");
-	    }
+	if (v3Global.assertDTypesResolved()) {
 	    if (!nodep->width() && nodep->castNodeMath()) {
-		nodep->v3fatalSrc("Math node has no assigned width\n");
+		nodep->v3fatalSrc("Math node has no assigned width");
+	    }
+	}
+	if (v3Global.assertWidthsMatch()) {
+	    if (nodep->width() != nodep->widthMin()) {
+		nodep->v3fatalSrc("Width != WidthMin");
 	    }
 	}
 	nodep->iterateChildren(*this);
