@@ -8,7 +8,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2008-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2008-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -862,9 +862,9 @@ class GaterVisitor : public GaterBaseVisitor {
 	    m_stmtVscp = NULL;
 	    m_stmtInPli = false;
 	}
-	if (!nodep->isSplittable()) {
+	if (!nodep->isPure() || nodep->isBrancher()) {
 	    // May also be a new statement (above if); if so we mark it immediately
-	    UINFO(9,"         NotSplittable "<<nodep<<endl);
+	    UINFO(9,"         NotPure "<<nodep<<endl);
 	    scoreboardPli(nodep);
 	}
 	{
