@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -322,7 +322,7 @@ public:
 	    AstVarRef* varrefp = nodep->memp()->castVarRef();
 	    if (!varrefp) { nodep->v3error("Readmem loading non-variable"); }
 	    else if (AstArrayDType* adtypep = varrefp->varp()->dtypeSkipRefp()->castArrayDType()) {
-		puts(cvtToStr(varrefp->varp()->arrayElements()));
+		puts(cvtToStr(varrefp->varp()->dtypep()->arrayElements()));
 		array_lsb = adtypep->lsb();
 	    }
 	    else {
@@ -552,7 +552,7 @@ public:
 	    nodep->v3error("Unsupported: 4-state numbers in this context");
 	} else if (nodep->isWide()) {
 	    putbs("VL_CONST_W_");
-	    puts(cvtToStr(VL_WORDS_I(nodep->num().minWidth())));
+	    puts(cvtToStr(VL_WORDS_I(nodep->num().widthMin())));
 	    puts("X(");
 	    puts(cvtToStr(nodep->widthMin()));
 	    puts(",");
@@ -564,7 +564,7 @@ public:
 	    } else {
 		assigntop->iterateAndNext(*this);
 	    }
-	    for (int word=VL_WORDS_I(nodep->num().minWidth())-1; word>0; word--) {
+	    for (int word=VL_WORDS_I(nodep->num().widthMin())-1; word>0; word--) {
 		// Only 32 bits - llx + long long here just to appease CPP format warning
 		ofp()->printf(",0x%08" VL_PRI64 "x", (vluint64_t)(nodep->num().dataWord(word)));
 	    }
@@ -677,10 +677,10 @@ class EmitCImp : EmitCStmts {
 		    puts(" | (");
 		}
 		changep->lhsp()->iterateAndNext(*this);
-		if (changep->isWide()) puts("["+cvtToStr(word)+"]");
+		if (changep->lhsp()->isWide()) puts("["+cvtToStr(word)+"]");
 		puts(" ^ ");
 		changep->rhsp()->iterateAndNext(*this);
-		if (changep->isWide()) puts("["+cvtToStr(word)+"]");
+		if (changep->lhsp()->isWide()) puts("["+cvtToStr(word)+"]");
 		puts(")");
 	    }
 	}
@@ -872,9 +872,6 @@ void EmitCStmts::emitVarDecl(AstVar* nodep, const string& prefixIfImp) {
     if (nodep->isIO()) {
 	bool isArray = !nodep->dtypeSkipRefp()->castBasicDType();
 	if (nodep->isSc()) {
-	    if (isArray) {
-		nodep->v3error("Unsupported: SystemC inputs and outputs must be simple data types; no arrays");
-	    }
 	    m_ctorVarsVec.push_back(nodep);
 	    ofp()->putAlign(nodep->isStatic(), 4);	// sc stuff is a structure, so bigger alignment
 	    if (nodep->attrScClocked() && nodep->isInput()) {
@@ -889,6 +886,12 @@ void EmitCStmts::emitVarDecl(AstVar* nodep, const string& prefixIfImp) {
 		puts(">\t");
 	    }
 	    puts(nodep->name());
+	    if (isArray) {
+		for (AstArrayDType* arrayp=nodep->dtypeSkipRefp()->castArrayDType(); arrayp;
+		     arrayp = arrayp->dtypeSkipRefp()->castArrayDType()) {
+		    puts("["+cvtToStr(arrayp->elementsConst())+"]");
+		}
+	    }
 	    puts(";\n");
 	} else { // C++ signals
 	    ofp()->putAlign(nodep->isStatic(), nodep->dtypeSkipRefp()->widthAlignBytes(),
@@ -905,7 +908,8 @@ void EmitCStmts::emitVarDecl(AstVar* nodep, const string& prefixIfImp) {
 	    if (isArray) {
 		if (nodep->isWide()) puts("W");
 		puts("("+nodep->name());
-		for (AstArrayDType* arrayp=nodep->dtypeSkipRefp()->castArrayDType(); arrayp; arrayp = arrayp->dtypeSkipRefp()->castArrayDType()) {
+		for (AstArrayDType* arrayp=nodep->dtypeSkipRefp()->castArrayDType(); arrayp;
+		     arrayp = arrayp->dtypeSkipRefp()->castArrayDType()) {
 		    puts("["+cvtToStr(arrayp->elementsConst())+"]");
 		}
 		puts(","+cvtToStr(basicp->msb())+","+cvtToStr(basicp->lsb()));
@@ -926,7 +930,8 @@ void EmitCStmts::emitVarDecl(AstVar* nodep, const string& prefixIfImp) {
 	// strings and other fundamental c types
 	puts(nodep->vlArgType(true,false));
 	// This isn't very robust and may need cleanup for other data types
-	for (AstArrayDType* arrayp=nodep->dtypeSkipRefp()->castArrayDType(); arrayp; arrayp = arrayp->dtypeSkipRefp()->castArrayDType()) {
+	for (AstArrayDType* arrayp=nodep->dtypeSkipRefp()->castArrayDType(); arrayp;
+	     arrayp = arrayp->dtypeSkipRefp()->castArrayDType()) {
 	    puts("["+cvtToStr(arrayp->elementsConst())+"]");
 	}
 	puts(";\n");
@@ -951,7 +956,8 @@ void EmitCStmts::emitVarDecl(AstVar* nodep, const string& prefixIfImp) {
 	if (prefixIfImp!="") { puts(prefixIfImp); puts("::"); }
 	puts(nodep->name());
 	// This isn't very robust and may need cleanup for other data types
-	for (AstArrayDType* arrayp=nodep->dtypeSkipRefp()->castArrayDType(); arrayp; arrayp = arrayp->dtypeSkipRefp()->castArrayDType()) {
+	for (AstArrayDType* arrayp=nodep->dtypeSkipRefp()->castArrayDType(); arrayp;
+	     arrayp = arrayp->dtypeSkipRefp()->castArrayDType()) {
 	    puts("["+cvtToStr(arrayp->elementsConst())+"]");
 	}
 	puts(","+cvtToStr(basicp->msb())+","+cvtToStr(basicp->lsb()));
@@ -961,22 +967,29 @@ void EmitCStmts::emitVarDecl(AstVar* nodep, const string& prefixIfImp) {
 }
 
 void EmitCStmts::emitVarCtors() {
-    ofp()->indentInc();
-    bool first = true;
-    for (vector<AstVar*>::iterator it = m_ctorVarsVec.begin(); it != m_ctorVarsVec.end(); ++it) {
-	if (first) {
-	    first=false;
-	    puts("\n");
-	    puts("#if (SYSTEMC_VERSION>20011000)\n");  // SystemC 2.0.1 and newer
-	    puts("  : ");
+    if (!m_ctorVarsVec.empty()) {
+	ofp()->indentInc();
+	puts("\n");
+	puts("#if (SYSTEMC_VERSION>20011000)\n");  // SystemC 2.0.1 and newer
+	bool first = true;
+	for (vector<AstVar*>::iterator it = m_ctorVarsVec.begin(); it != m_ctorVarsVec.end(); ++it) {
+	    AstVar* varp = *it;
+	    bool isArray = !varp->dtypeSkipRefp()->castBasicDType();
+	    if (isArray) {
+		puts("// Skipping array: ");
+		puts(varp->name());
+		puts("\n");
+	    } else {
+		if (first) { puts("  : "); first=false; }
+		else puts(", ");
+		if (ofp()->exceededWidth()) puts("\n  ");
+		puts(varp->name());
+		puts("("); putsQuoted(varp->name()); puts(")");
+	    }
 	}
-	else puts(", ");
-	if (ofp()->exceededWidth()) puts("\n  ");
-	puts((*it)->name());
-	puts("("); putsQuoted((*it)->name()); puts(")");
+	puts ("\n#endif\n");
+	ofp()->indentDec();
     }
-    if (!first) puts ("\n#endif\n");
-    ofp()->indentDec();
 }
 
 bool EmitCStmts::emitSimpleOk(AstNodeMath* nodep) {
@@ -1353,7 +1366,7 @@ void EmitCImp::emitVarResets(AstNodeModule* modp) {
 		    puts(varp->name());
 		    for (int v=0; v<vects; ++v) puts( "[__Vi"+cvtToStr(v)+"]");
 		    if (zeroit) {
-			puts("= 0;\n");
+			puts(" = 0;\n");
 		    } else {
 			puts(" = VL_RAND_RESET_");
 			emitIQW(varp);
@@ -1500,7 +1513,22 @@ void EmitCImp::emitSensitives() {
 	for (AstNode* nodep=m_modp->stmtsp(); nodep; nodep = nodep->nextp()) {
 	    if (AstVar* varp = nodep->castVar()) {
 		if (varp->isInput() && (varp->isScSensitive() || varp->isUsedClock())) {
-		    puts("sensitive << "+varp->name()+";\n");
+		    int vects = 0;
+		    // This isn't very robust and may need cleanup for other data types
+		    for (AstArrayDType* arrayp=varp->dtypeSkipRefp()->castArrayDType(); arrayp;
+			 arrayp = arrayp->dtypeSkipRefp()->castArrayDType()) {
+			int vecnum = vects++;
+			if (arrayp->msb() < arrayp->lsb()) varp->v3fatalSrc("Should have swapped msb & lsb earlier.");
+			string ivar = string("__Vi")+cvtToStr(vecnum);
+			// MSVC++ pre V7 doesn't support 'for (int ...)', so declare in sep block
+			puts("{ int __Vi"+cvtToStr(vecnum)+"="+cvtToStr(arrayp->lsb())+";");
+			puts(" for (; "+ivar+"<="+cvtToStr(arrayp->msb()));
+			puts("; ++"+ivar+") {\n");
+		    }
+		    puts("sensitive << "+varp->name());
+		    for (int v=0; v<vects; ++v) puts( "[__Vi"+cvtToStr(v)+"]");
+		    puts(";\n");
+		    for (int v=0; v<vects; ++v) puts( "}}\n");
 		}
 	    }
 	}
@@ -2062,7 +2090,8 @@ class EmitCTrace : EmitCStmts {
 	} else {
 	    puts(",-1");
 	}
-	if (nodep->msbEndianed() || nodep->lsbEndianed()) {
+	if (!nodep->isDouble()  // When float/double no longer have widths this can go
+	    && (nodep->msbEndianed() || nodep->lsbEndianed())) {
 	    puts(","+cvtToStr(nodep->msbEndianed())+","+cvtToStr(nodep->lsbEndianed()));
 	}
 	puts(");");
@@ -2088,7 +2117,8 @@ class EmitCTrace : EmitCStmts {
 			    + ((arrayindex<0) ? 0 : (arrayindex*nodep->declp()->widthWords()))));
 	puts(",");
 	emitTraceValue(nodep, arrayindex);
-	if (nodep->declp()->msbEndianed() || nodep->declp()->lsbEndianed() || emitTraceIsScBv(nodep)) {
+	if (!nodep->isDouble()  // When float/double no longer have widths this can go
+	    && (nodep->declp()->msbEndianed() || nodep->declp()->lsbEndianed() || emitTraceIsScBv(nodep))) {
 	    puts(","+cvtToStr(nodep->declp()->widthMin()));
 	}
 	puts(");\n");
