@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -48,11 +48,12 @@ class EmitCInlines : EmitCBaseVisitor {
     virtual void visit(AstVar* nodep, AstNUser*) {
 	// All wide constants load into variables, so we can just hunt for them
 	nodep->iterateChildren(*this);
-	if (nodep->widthWords() >= EMITCINLINES_NUM_CONSTW ) {
-	    if (int(m_wordWidths.size()) <= nodep->widthWords()) {
-		m_wordWidths.resize(nodep->widthWords()+5);
+	int words = nodep->widthWords();
+	if (words >= EMITCINLINES_NUM_CONSTW ) {
+	    if (int(m_wordWidths.size()) <= words) {
+		m_wordWidths.resize(words+5);
 	    }
-	    ++ m_wordWidths.at(nodep->widthWords());
+	    ++ m_wordWidths.at(words);
 	    v3Global.needHInlines(true);
 	}
     }
