@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -453,9 +453,9 @@ void EmitCSyms::emitSymImp() {
 	    string bounds;
 	    if (AstBasicDType* basicp = varp->basicp()) {
 		// Range is always first, it's not in "C" order
-		if (basicp->rangep()) {
-		    bounds += " ,"; bounds += cvtToStr(basicp->rangep()->msbConst());
-		    bounds += ","; bounds += cvtToStr(basicp->rangep()->lsbConst());
+		if (basicp->isRanged()) {
+		    bounds += " ,"; bounds += cvtToStr(basicp->msb());
+		    bounds += ","; bounds += cvtToStr(basicp->lsb());
 		    dim++;
 		}
 		for (AstNodeDType* dtypep=varp->dtypep(); dtypep; ) {
