@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2004-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2004-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -82,7 +82,11 @@ class EmitVBaseVisitor : public EmitCBaseVisitor {
     }
 
     virtual void visit(AstBegin* nodep, AstNUser*) {
-	putbs("begin\n");
+	if (nodep->unnamed()) {
+	    putbs("begin\n");
+	} else {
+	    putbs("begin : "+nodep->name()+"\n");
+	}
 	nodep->iterateChildren(*this);
 	puts("end\n");
     }
@@ -482,6 +486,7 @@ class EmitVBaseVisitor : public EmitCBaseVisitor {
 	if (nodep->isSigned()) putfs(nodep,"signed ");
 	putfs(nodep,nodep->prettyName());
 	if (nodep->rangep()) { puts(" "); nodep->rangep()->iterateAndNext(*this); puts(" "); }
+	else if (nodep->isRanged()) { puts(" ["); puts(cvtToStr(nodep->msb())); puts(":0] "); }
     }
     virtual void visit(AstConstDType* nodep, AstNUser*) {
 	putfs(nodep,"const ");
