@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -55,7 +55,7 @@ v3errorIniter v3errorInit;
 
 V3ErrorCode::V3ErrorCode(const char* msgp) {
     // Return error encoding for given string, or ERROR, which is a bad code
-    for (int codei=V3ErrorCode::EC_FIRST_WARN; codei<V3ErrorCode::_ENUM_MAX; codei++) {
+    for (int codei=V3ErrorCode::EC_MIN; codei<V3ErrorCode::_ENUM_MAX; codei++) {
 	V3ErrorCode code = (V3ErrorCode)codei;
 	if (0==strcasecmp(msgp,code.ascii())) {
 	    m_e = code; return;
@@ -204,6 +204,9 @@ bool FileLine::warnOff(const string& msg, bool flag) {
     V3ErrorCode code (msg.c_str());
     if (code < V3ErrorCode::EC_FIRST_WARN) {
 	return false;
+    } else if (v3Global.opt.lintOnly()   // Lint mode is allowed to suppress some errors
+	       && code < V3ErrorCode::EC_MIN) {
+	return false;
     } else {
 	warnOff(code, flag);
 	return true;
@@ -211,14 +214,14 @@ bool FileLine::warnOff(const string& msg, bool flag) {
 }
 
 void FileLine::warnLintOff(bool flag) {
-    for (int codei=V3ErrorCode::EC_FIRST_WARN; codei<V3ErrorCode::_ENUM_MAX; codei++) {
+    for (int codei=V3ErrorCode::EC_MIN; codei<V3ErrorCode::_ENUM_MAX; codei++) {
 	V3ErrorCode code = (V3ErrorCode)codei;
 	if (code.lintError()) warnOff(code, flag);
     }
 }
 
 void FileLine::warnStyleOff(bool flag) {
-    for (int codei=V3ErrorCode::EC_FIRST_WARN; codei<V3ErrorCode::_ENUM_MAX; codei++) {
+    for (int codei=V3ErrorCode::EC_MIN; codei<V3ErrorCode::_ENUM_MAX; codei++) {
 	V3ErrorCode code = (V3ErrorCode)codei;
 	if (code.styleError()) warnOff(code, flag);
     }
@@ -234,7 +237,7 @@ bool FileLine::warnIsOff(V3ErrorCode code) const {
 
 void FileLine::modifyStateInherit(const FileLine* fromp) {
     // Any warnings that are off in "from", become off in "this".
-    for (int codei=V3ErrorCode::EC_FIRST_WARN; codei<V3ErrorCode::_ENUM_MAX; codei++) {
+    for (int codei=V3ErrorCode::EC_MIN; codei<V3ErrorCode::_ENUM_MAX; codei++) {
 	V3ErrorCode code = (V3ErrorCode)codei;
 	if (fromp->warnIsOff(code)) {
 	    this->warnOff(code, true);
@@ -396,7 +399,7 @@ string V3Error::v3sform (const char* format, ...) {
 }
 
 void V3Error::suppressThisWarning() {
-    if (s_errorCode>=V3ErrorCode::EC_FIRST_WARN) {
+    if (s_errorCode>=V3ErrorCode::EC_MIN) {
 	V3Stats::addStatSum(string("Warnings, Suppressed ")+s_errorCode.ascii(), 1);
 	s_errorSuppressed = true;
     }
