@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -195,7 +195,8 @@ private:
 	// *** Special iterator
 	if (!m_isSimple) return;	// Fastpath
 	if (!nodep->isGateOptimizable()
-	    || !nodep->isSplittable()) {
+	    || !nodep->isPure()
+	    || nodep->isBrancher()) {
 	    UINFO(5, "Non optimizable type: "<<nodep<<endl);
 	    clearSimple("Non optimizable type");
 	}
