@@ -8,7 +8,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -29,7 +29,8 @@
 
 #include "V3Error.h"
 #include "V3Options.h"
-#include "V3Ast.h"
+
+class AstNetlist;
 
 //======================================================================
 // Statics
@@ -40,9 +41,11 @@
 
 class V3Global {
     // Globals
-    AstNetlist*	m_rootp;	// Root of entire netlist
+    AstNetlist*	m_rootp;		// Root of entire netlist
+
     int		m_debugFileNumber;	// Number to append to debug files created
-    bool	m_assertWidthsSame;	// Tree should have width()==widthMin()
+    bool	m_assertDTypesResolved;	// Tree should have dtypep()'s
+    bool	m_assertWidthsMatch;	// Tree should have width()==widthMin()
     bool	m_needHInlines;		// Need __Inlines file
     bool	m_needHeavy;		// Need verilated_heavy.h include
     bool	m_dpi;			// Need __Dpi include files
@@ -54,24 +57,26 @@ public:
   public:
     // CREATORS
     V3Global() {
-	m_rootp = new AstNetlist;
 	m_debugFileNumber = 0;
-	m_assertWidthsSame = false;
+	m_assertDTypesResolved = false;
+	m_assertWidthsMatch = false;
 	m_needHInlines = false;
 	m_needHeavy = false;
 	m_dpi = false;
+	m_rootp = makeNetlist();
     }
-    void clear() {
-	if (m_rootp) m_rootp->deleteTree(); m_rootp=NULL;
-    }
+    AstNetlist* makeNetlist();
+    void clear();
     // ACCESSORS (general)
     AstNetlist* rootp() const { return m_rootp; }
-    bool assertWidthsSame() const { return m_assertWidthsSame; }
+    bool assertDTypesResolved() const { return m_assertDTypesResolved; }
+    bool assertWidthsMatch() const { return m_assertWidthsMatch; }
 
     // METHODS
     void readFiles();
-    void checkTree() { rootp()->checkTree(); }
-    void assertWidthsSame(bool flag) { m_assertWidthsSame = flag; }
+    void checkTree();
+    void assertDTypesResolved(bool flag) { m_assertDTypesResolved = flag; }
+    void assertWidthsMatch(bool flag) { m_assertWidthsMatch = flag; }
     string debugFilename(const string& nameComment, int newNumber=0) {
 	++m_debugFileNumber;
 	if (newNumber) m_debugFileNumber = newNumber;
