@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -199,12 +199,14 @@ private:
 	    int expwidth = nodep->exprp()->width();
 	    if (expwidth == pinwidth) {
 		// NOP: Arrayed instants: widths match so connect to each instance
-	    } else if (expwidth == pinwidth*m_cellRangep->width()) {
+	    } else if (expwidth == pinwidth*m_cellRangep->elementsConst()) {
 		// Arrayed instants: one bit for each of the instants (each assign is 1 pinwidth wide)
 		AstNode* exprp = nodep->exprp()->unlinkFrBack();
 		bool inputPin = nodep->modVarp()->isInput();
-		if (!inputPin && !exprp->castVarRef()) {
+		if (!inputPin && !exprp->castVarRef()
+		    && !exprp->castSel()) {  // V3Const will collapse the SEL with the one we're about to make
 		    nodep->v3error("Unsupported: Per-bit array instantiations with output connections to non-wires.");
+		    // Note spec allows more complicated matches such as slices and such
 		}
 		exprp = new AstSel (exprp->fileline(), exprp,
 				    pinwidth*(m_instNum-m_instLsb),
@@ -282,7 +284,7 @@ void V3Inst::pinReconnectSimple(AstPin* pinp, AstCell* cellp, AstNodeModule* mod
 		    rhsp = new AstExtend (pinp->fileline(), rhsp);
 		}
 	    } else if (pinp->width() < rhsp->width()) {
-		rhsp = new AstSel    (pinp->fileline(), rhsp, 0, pinp->widthMin());
+		rhsp = new AstSel    (pinp->fileline(), rhsp, 0, pinp->width());
 	    }
 	    rhsp->widthSignedFrom(pinp);
 	    assignp = new AstAssignW (pinp->fileline(), pinexprp, rhsp);
