@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -79,6 +79,7 @@ private:
     V3SymTable* m_cellVarsp;	// Symbol table of variables under cell's module
     int		m_beginNum;	// Begin block number, 0=none seen
     int		m_modBeginNum;	// Begin block number in module, 0=none seen
+    bool	m_inAlways;	// Inside an always
     bool	m_inGenerate;	// Inside a generate
     AstNodeModule*	m_valueModp;	// If set, move AstVar->valuep() initial values to this module
     vector<V3SymTable*> m_delSymps;	// Symbol tables to delete
@@ -319,7 +320,8 @@ private:
 		    }
 		} else {
 		    // User can disable the message at either point
-		    if (!nodep->fileline()->warnIsOff(V3ErrorCode::VARHIDDEN)
+		    if (!(m_ftaskp && m_ftaskp->dpiImport())
+			&& !nodep->fileline()->warnIsOff(V3ErrorCode::VARHIDDEN)
 			&& !foundp->fileline()->warnIsOff(V3ErrorCode::VARHIDDEN)) {
 			nodep->v3warn(VARHIDDEN,"Declaration of signal hides declaration in upper scope: "<<nodep->name());
 			foundp->v3warn(VARHIDDEN,"... Location of original declaration");
@@ -339,6 +341,8 @@ private:
 	    if (nodep->isIO() && !m_ftaskp && !nodep->user2()) {
 		nodep->v3error("Input/output/inout does not appear in port list: "<<nodep->prettyName());
 	    }
+	    // temporaries under an always aren't expected to be blocking
+	    if (m_inAlways) nodep->fileline()->modifyWarnOff(V3ErrorCode::BLKSEQ, true);
 	    if (nodep->valuep()) {
 		// A variable with a = value can be three things:
 		FileLine* fl = nodep->valuep()->fileline();
@@ -478,7 +482,7 @@ private:
 	}
 	// Recurse
 	int oldNum = m_beginNum;
-	m_beginNum = 0;
+	if (!nodep->hidden()) m_beginNum = 0;
 	{
 	    // Create symbol table for the task's vars
 	    m_curVarsp = symsFindNew(nodep, upperVarsp);
@@ -546,7 +550,7 @@ private:
 		defp = m_curVarsp->findIdUpward(nodep->name())->castTypedef();
 	    }
 	    if (!defp) { nodep->v3error("Can't find typedef: "<<nodep->prettyName()); }
-	    nodep->defp(defp);
+	    nodep->defp(defp->dtypep());
 	    nodep->packagep(packageFor(defp));
 	}
 	nodep->iterateChildren(*this);
@@ -744,7 +748,9 @@ private:
 	visitIterateNoValueMod(nodep);
     }
     virtual void visit(AstAlways* nodep, AstNUser*) {
+	m_inAlways = true;
 	visitIterateNoValueMod(nodep);
+	m_inAlways = false;
     }
     virtual void visit(AstPslCover* nodep, AstNUser*) {
 	visitIterateNoValueMod(nodep);
@@ -767,6 +773,7 @@ public:
 	m_paramNum = 0;
 	m_beginNum = 0;
 	m_modBeginNum = 0;
+	m_inAlways = false;
 	m_inGenerate = false;
 	m_valueModp = NULL;
 	//
