@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -479,13 +479,14 @@ private:
 	LinkDotBaseVertex* oldVxp = m_inlineVxp;
 	{
 	    m_beginp = nodep;
-	    // Ignore begin names
+	    // We don't pickup variables (as not supported yet), but do need to find cells
 	    m_inlineVxp = m_statep->insertBegin(m_inlineVxp, m_cellVxp, nodep);
-	    // We don't pickup variables, but do need to find cells
-	    nodep->iterateChildren(*this);
+	    nodep->stmtsp()->iterateAndNext(*this);
 	}
 	m_inlineVxp = oldVxp;
 	m_beginp = oldbegin;
+	//
+	nodep->flatsp()->iterateAndNext(*this);
     }
     virtual void visit(AstVar* nodep, AstNUser*) {
 	if (!m_statep->forScopeCreation()
@@ -497,7 +498,9 @@ private:
 	}
     }
     virtual void visit(AstNodeFTask* nodep, AstNUser*) {
-	m_statep->insertSym(m_cellVxp, nodep->name(), nodep);
+	if (!m_beginp) {	// For now, we don't support xrefs into functions inside begin blocks
+	    m_statep->insertSym(m_cellVxp, nodep->name(), nodep);
+	}
 	// No recursion, we don't want to pick up variables
     }
     virtual void visit(AstCFunc* nodep, AstNUser*) {
@@ -680,6 +683,9 @@ private:
 	} else if (!m_cellVxp) {
 	    UINFO(9,"Dead module for "<<nodep<<endl);
 	    nodep->taskp(NULL);  // Module that is not in hierarchy.  We'll be dead code eliminating it later.
+	} else if (nodep->dotted()=="" && nodep->taskp()) {
+	    // V3Link should have setup the links
+	    // Might be under a BEGIN we're not processing, so don't relink it
 	} else {
 	    string baddot;
 	    LinkDotBaseVertex* okVxp;
