@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -275,7 +275,6 @@ private:
 	}
     }
 
-
     virtual void visit(AstVar* nodep, AstNUser*) {
 	cleanFileline(nodep);
 	m_varp = nodep;
@@ -362,7 +361,7 @@ private:
 	    // Definition must be inserted right after the variable (etc) that needed it
 	    // AstVar, AstTypedef, AstNodeFTask are common containers
 	    AstNode* backp = nodep->backp();
-	    for (; backp; backp=nodep->backp()) {
+	    for (; backp; backp=backp->backp()) {
 		if (backp->castVar()) break;
 		else if (backp->castTypedef()) break;
 		else if (backp->castNodeFTask()) break;
