@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -626,7 +626,7 @@ bool V3Number::isLt(const V3Number& rhs) const {
     return 0;
 }
 
-int V3Number::minWidth() const {
+int V3Number::widthMin() const {
     for(int bit=width()-1; bit>0; bit--) {
 	if (!bitIs0(bit)) return bit+1;
     }
