@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -55,8 +55,8 @@ struct V3OptionsImp {
     set<string>		m_incDirUserSet;	// Include directories (for removing duplicates)
     list<string>	m_incDirFallbacks;	// Include directories (ordered)
     set<string>		m_incDirFallbackSet;	// Include directories (for removing duplicates)
-    list<string>	m_libExts;	// Library extensions (ordered)
-    set<string>		m_libExtSet;	// Library extensions (for removing duplicates)
+    list<string>	m_libExtVs;	// Library extensions (ordered)
+    set<string>		m_libExtVSet;	// Library extensions (for removing duplicates)
     DirMap		m_dirMap;	// Directory listing
 
     // ACCESSOR METHODS
@@ -76,10 +76,10 @@ struct V3OptionsImp {
 	    }
 	}
     }
-    void addLibExt(const string& libext) {
-	if (m_libExtSet.find(libext) == m_libExtSet.end()) {
-	    m_libExtSet.insert(libext);
-	    m_libExts.push_back(libext);
+    void addLibExtV(const string& libext) {
+	if (m_libExtVSet.find(libext) == m_libExtVSet.end()) {
+	    m_libExtVSet.insert(libext);
+	    m_libExtVs.push_back(libext);
 	}
     }
     V3OptionsImp() {}
@@ -91,8 +91,8 @@ void V3Options::addIncDirUser(const string& incdir) {
 void V3Options::addIncDirFallback(const string& incdir) {
     m_impp->addIncDirFallback(incdir);
 }
-void V3Options::addLibExt(const string& libext) {
-    m_impp->addLibExt(libext);
+void V3Options::addLibExtV(const string& libext) {
+    m_impp->addLibExtV(libext);
 }
 void V3Options::addDefine(const string& defline) {
     // Split +define+foo=value into the appropriate parts and parse
@@ -304,7 +304,7 @@ string V3Options::fileExists (const string& filename) {
 }
 
 string V3Options::filePathCheckOneDir(const string& modname, const string& dirname) {
-    for (list<string>::iterator extIter=m_impp->m_libExts.begin(); extIter!=m_impp->m_libExts.end(); ++extIter) {
+    for (list<string>::iterator extIter=m_impp->m_libExtVs.begin(); extIter!=m_impp->m_libExtVs.end(); ++extIter) {
 	string fn = filenameFromDirBase(dirname, modname+*extIter);
 	string exists = fileExists(fn);
 	if (exists!="") {
@@ -350,14 +350,14 @@ void V3Options::filePathLookedMsg(FileLine* fl, const string& modname) {
 	fl->v3error("Looked in:"<<endl);
 	for (list<string>::iterator dirIter=m_impp->m_incDirUsers.begin();
 	     dirIter!=m_impp->m_incDirUsers.end(); ++dirIter) {
-	    for (list<string>::iterator extIter=m_impp->m_libExts.begin(); extIter!=m_impp->m_libExts.end(); ++extIter) {
+	    for (list<string>::iterator extIter=m_impp->m_libExtVs.begin(); extIter!=m_impp->m_libExtVs.end(); ++extIter) {
 		string fn = filenameFromDirBase(*dirIter,modname+*extIter);
 		fl->v3error("      "<<fn<<endl);
 	    }
 	}
 	for (list<string>::iterator dirIter=m_impp->m_incDirFallbacks.begin();
 	     dirIter!=m_impp->m_incDirFallbacks.end(); ++dirIter) {
-	    for (list<string>::iterator extIter=m_impp->m_libExts.begin(); extIter!=m_impp->m_libExts.end(); ++extIter) {
+	    for (list<string>::iterator extIter=m_impp->m_libExtVs.begin(); extIter!=m_impp->m_libExtVs.end(); ++extIter) {
 		string fn = filenameFromDirBase(*dirIter,modname+*extIter);
 		fl->v3error("      "<<fn<<endl);
 	    }
@@ -414,13 +414,6 @@ string V3Options::getenvSYSTEMC() {
 	var = DEFENV_SYSTEMC;
 	setenvStr("SYSTEMC", var, "Hardcoded at build time");
     }
-    // Only correct or check it if we really need the value
-    if (v3Global.opt.usingSystemCLibs()) {
-	if (var == "") {
-	    v3fatal("Need $SYSTEMC in environment\n"
-		    "Probably System-C isn't installed, see http://www.systemc.org\n");
-	}
-    }
     return var;
 }
 
@@ -451,6 +444,47 @@ string V3Options::getenvSYSTEMC_ARCH() {
     return var;
 }
 
+string V3Options::getenvSYSTEMC_INCLUDE() {
+    string var = getenvStr("SYSTEMC_INCLUDE","");
+    if (var == "" && string(DEFENV_SYSTEMC_INCLUDE) != "") {
+	var = DEFENV_SYSTEMC_INCLUDE;
+	setenvStr("SYSTEMC_INCLUDE", var, "Hardcoded at build time");
+    }
+    if (var == "") {
+	string sc = getenvSYSTEMC();
+	if (sc != "") var = sc+"/include";
+    }
+    // Only correct or check it if we really need the value
+    if (v3Global.opt.usingSystemCLibs()) {
+	if (var == "") {
+	    v3fatal("Need $SYSTEMC_INCLUDE in environment or when Verilator configured\n"
+		    "Probably System-C isn't installed, see http://www.systemc.org\n");
+	}
+    }
+    return var;
+}
+
+string V3Options::getenvSYSTEMC_LIBDIR() {
+    string var = getenvStr("SYSTEMC_LIBDIR","");
+    if (var == "" && string(DEFENV_SYSTEMC_LIBDIR) != "") {
+	var = DEFENV_SYSTEMC_LIBDIR;
+	setenvStr("SYSTEMC_LIBDIR", var, "Hardcoded at build time");
+    }
+    if (var == "") {
+	string sc = getenvSYSTEMC();
+	string arch = getenvSYSTEMC_ARCH();
+	if (sc != "" && arch != "") var = sc+"/lib-"+arch;
+    }
+    // Only correct or check it if we really need the value
+    if (v3Global.opt.usingSystemCLibs()) {
+	if (var == "") {
+	    v3fatal("Need $SYSTEMC_LIBDIR in environment or when Verilator configured\n"
+		    "Probably System-C isn't installed, see http://www.systemc.org\n");
+	}
+    }
+    return var;
+}
+
 string V3Options::getenvSYSTEMPERL() {
     // Must be careful to set SYSTEMPERL_INCLUDE first else we'd setenv
     // SYSTEMPERL which would override a DEFENVed SYSTEMPERL_INCLUDE.
@@ -651,10 +685,10 @@ void V3Options::parseOptsList(FileLine* fl, const string& optdir, int argc, char
 		string exts = string(sw+strlen("+libext+"));
 		string::size_type pos;
 		while ((pos=exts.find("+")) != string::npos) {
-		    addLibExt (exts.substr(0,pos));
+		    addLibExtV (exts.substr(0,pos));
 		    exts = exts.substr(pos+1);
 		}
-		addLibExt (exts);
+		addLibExtV (exts);
 	    }
 	    else if ( !strcmp (sw, "+librescan")) { // NOP
 	    }
@@ -1083,7 +1117,7 @@ void V3Options::showVersion(bool verbose) {
     if (!verbose) return;
 
     cout <<endl;
-    cout << "Copyright 2003-2011 by Wilson Snyder.  Verilator is free software; you can\n";
+    cout << "Copyright 2003-2012 by Wilson Snyder.  Verilator is free software; you can\n";
     cout << "redistribute it and/or modify the Verilator internals under the terms of\n";
     cout << "either the GNU Lesser General Public License Version 3 or the Perl Artistic\n";
     cout << "License Version 2.0.\n";
@@ -1096,6 +1130,8 @@ void V3Options::showVersion(bool verbose) {
     cout << "  Compiled in defaults if not in environment:\n";
     cout << "    SYSTEMC            = " << DEFENV_SYSTEMC<<endl;
     cout << "    SYSTEMC_ARCH       = " << DEFENV_SYSTEMC_ARCH<<endl;
+    cout << "    SYSTEMC_INCLUDE    = " << DEFENV_SYSTEMC_INCLUDE<<endl;
+    cout << "    SYSTEMC_LIBDIR     = " << DEFENV_SYSTEMC_LIBDIR<<endl;
     cout << "    SYSTEMPERL         = " << DEFENV_SYSTEMPERL<<endl;
     cout << "    SYSTEMPERL_INCLUDE = " << DEFENV_SYSTEMPERL_INCLUDE<<endl;
     cout << "    VERILATOR_ROOT     = " << DEFENV_VERILATOR_ROOT<<endl;
@@ -1105,6 +1141,8 @@ void V3Options::showVersion(bool verbose) {
     cout << "    PERL               = " << getenvStr("PERL","")<<endl;
     cout << "    SYSTEMC            = " << getenvStr("SYSTEMC","")<<endl;
     cout << "    SYSTEMC_ARCH       = " << getenvStr("SYSTEMC_ARCH","")<<endl;
+    cout << "    SYSTEMC_INCLUDE    = " << getenvStr("SYSTEMC_INCLUDE","")<<endl;
+    cout << "    SYSTEMC_LIBDIR     = " << getenvStr("SYSTEMC_LIBDIR","")<<endl;
     cout << "    SYSTEMPERL         = " << getenvStr("SYSTEMPERL","")<<endl;
     cout << "    SYSTEMPERL_INCLUDE = " << getenvStr("SYSTEMPERL_INCLUDE","")<<endl;
     cout << "    VERILATOR_ROOT     = " << getenvStr("VERILATOR_ROOT","")<<endl;
@@ -1170,9 +1208,9 @@ V3Options::V3Options() {
 
     optimize(true);
     // Default +libext+
-    addLibExt("");  // So include "filename.v" will find the same file
-    addLibExt(".v");
-    addLibExt(".sv");
+    addLibExtV("");  // So include "filename.v" will find the same file
+    addLibExtV(".v");
+    addLibExtV(".sv");
     // Default -I
     addIncDirFallback(".");	// Looks better than {long_cwd_path}/...
 }
