@@ -8,7 +8,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -183,7 +183,7 @@ class V3Options {
     void addFuture(const string& flag);
     void addIncDirUser(const string& incdir);  // User requested
     void addIncDirFallback(const string& incdir);  // Low priority if not found otherwise
-    void addLibExt(const string& libext);
+    void addLibExtV(const string& libext);
     void optimize(int level);
     void showVersion(bool verbose);
     void coverage(bool flag) { m_coverageLine = m_coverageToggle = m_coverageUser = flag; }
@@ -340,6 +340,8 @@ class V3Options {
     static string getenvPERL();
     static string getenvSYSTEMC();
     static string getenvSYSTEMC_ARCH();
+    static string getenvSYSTEMC_INCLUDE();
+    static string getenvSYSTEMC_LIBDIR();
     static string getenvSYSTEMPERL();
     static string getenvSYSTEMPERL_INCLUDE();
     static string getenvVERILATOR_ROOT();
