@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -204,6 +204,7 @@ private:
 		AstNRelinker replaceHandle;
 		nodep->unlinkFrBack(&replaceHandle);
 		AstNode* constzerop;
+		int m1value = nodep->widthMin()-1; // Constant of width-1; not changing dtype width
 		if (nodep->signedFlavor()) {
 		    // Then over shifting gives the sign bit, not all zeros
 		    // Note *NOT* clean output -- just like normal shift!
@@ -212,7 +213,7 @@ private:
 						new AstShiftR(nodep->fileline(),
 							      nodep->lhsp()->cloneTree(false),
 							      new AstConst(nodep->fileline(),
-									   nodep->widthMin()-1),
+									   m1value),
 							      nodep->width()));
 		} else {
 		    V3Number zeronum  (nodep->fileline(), nodep->width(), 0);
@@ -229,7 +230,6 @@ private:
 					     constwidthp),
 				 nodep,
 				 constzerop);
-		newp->widthSignedFrom(nodep);
 		replaceHandle.relink(newp);
 	    }
 	}
