@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -46,7 +46,7 @@
 //			  v1Dly -> S1 -> {empty}
 //  Multiple assignments to the same variable must remain in order
 //
-//  Also vars must not be "public" and we also scoreboard nodep->isSplittable()
+//  Also vars must not be "public" and we also scoreboard nodep->isPure()
 //
 //*************************************************************************
 
@@ -555,7 +555,7 @@ private:
     // Default
     virtual void visit(AstNode* nodep, AstNUser*) {
 	// **** SPECIAL default type that sets PLI_ORDERING
-	if (!m_stmtStackps.empty() && !nodep->isSplittable()) {
+	if (!m_stmtStackps.empty() && !nodep->isPure()) {
 	    UINFO(9,"         NotSplittable "<<nodep<<endl);
 	    scoreboardPli();
 	}
