@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -182,19 +182,21 @@ private:
 	// Index into our table
 	AstVar* indexVarp = new AstVar (nodep->fileline(), AstVarType::BLOCKTEMP,
 					"__Vtableidx" + cvtToStr(m_modTables),
-					AstLogicPacked(), m_inWidth);
+					AstBitPacked(), m_inWidth);
 	m_modp->addStmtp(indexVarp);
 	AstVarScope* indexVscp = new AstVarScope (indexVarp->fileline(), m_scopep, indexVarp);
 	m_scopep->addVarp(indexVscp);
 
 	// Change it variable
 	FileLine* fl = nodep->fileline();
+	AstNodeDType* dtypep
+	    = new AstArrayDType (fl,
+				 new AstBasicDType(fl, AstBitPacked(), m_outVarps.size()),
+				 new AstRange (fl, VL_MASK_I(m_inWidth), 0));
 	AstVar* chgVarp
 	    = new AstVar (fl, AstVarType::MODULETEMP,
 			  "__Vtablechg" + cvtToStr(m_modTables),
-			  new AstArrayDType (fl,
-					     new AstBasicDType(fl, AstLogicPacked(), m_outVarps.size()),
-					     new AstRange (fl, VL_MASK_I(m_inWidth), 0)));
+			  dtypep);
 	chgVarp->isConst(true);
 	chgVarp->valuep(new AstInitArray (nodep->fileline(), NULL));
 	m_modp->addStmtp(chgVarp);
@@ -233,12 +235,15 @@ private:
 	    AstVarScope* outvscp = *it;
 	    AstVar* outvarp = outvscp->varp();
 	    FileLine* fl = nodep->fileline();
+	    AstNodeDType* dtypep
+		= new AstArrayDType (fl,
+				     // FUTURE: If support more types, below can use outvarp->dtype()
+				     new AstBasicDType(fl, AstLogicPacked(), outvarp->width()),
+				     new AstRange (fl, VL_MASK_I(m_inWidth), 0));
 	    AstVar* tablevarp
 		= new AstVar (fl, AstVarType::MODULETEMP,
 			      "__Vtable" + cvtToStr(m_modTables) +"_"+outvarp->name(),
-			      new AstArrayDType (fl,
-						 new AstBasicDType(fl, AstLogicPacked(), outvarp->widthMin()),
-						 new AstRange (fl, VL_MASK_I(m_inWidth), 0)));
+			      dtypep);
 	    tablevarp->isConst(true);
 	    tablevarp->isStatic(true);
 	    tablevarp->valuep(new AstInitArray (nodep->fileline(), NULL));
@@ -344,7 +349,7 @@ private:
 	    AstVarScope* vsc2p= *it;
 	    AstVar* var2p = vsc2p->varp();
 	    if (var1p->width() == var2p->width()
-		&& var1p->arrayElements() == var2p->arrayElements()) {
+		&& var1p->dtypep()->arrayElements() == var2p->dtypep()->arrayElements()) {
 		AstNode* init1p = var1p->valuep()->castInitArray();
 		AstNode* init2p = var2p->valuep()->castInitArray();
 		if (init1p->sameTree(init2p)) {
