@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -223,7 +223,7 @@ private:
 	nodep->iterateChildren(*this);
 	if (nodep->varp()->user4p() != m_curVxp) {
 	    if (m_curVxp->pure()
-		&& !nodep->varp()->isPure()) {
+		&& !nodep->varp()->isXTemp()) {
 		m_curVxp->impure(nodep);
 	    }
 	}
@@ -621,13 +621,13 @@ private:
 	    stmt += "VL_CVT_VP_Q(";
 	    ket += ")";
 	}
-	else if (portp->basicp() && portp->basicp()->isBitLogic() && portp->widthMin() != 1 && portp->isQuad()) {
+	else if (portp->basicp() && portp->basicp()->isBitLogic() && portp->width() != 1 && portp->isQuad()) {
 	    // SV is vector, Verilator isn't
 	    stmt += "VL_SET_QW(";
 	    ket += ")";
 	}
 	if (!cvt
-	    && portp->basicp() && portp->basicp()->isBitLogic() && portp->widthMin() != 1 && !portp->isWide())
+	    && portp->basicp() && portp->basicp()->isBitLogic() && portp->width() != 1 && !portp->isWide() && !portp->isQuad())
 	    stmt += "*";  // it's a svBitVecVal, which other code won't think is arrayed (as WData aren't), but really is
 	stmt += frName;
 	stmt += ket;
@@ -778,7 +778,7 @@ private:
 		    if (args != "") { args+= ", "; }
 		    if (bitvec) {}
 		    else if (portp->isOutput()) args += "&";
-		    else if (portp->basicp() && portp->basicp()->isBitLogic() && portp->widthMin() != 1) args += "&";  // it's a svBitVecVal
+		    else if (portp->basicp() && portp->basicp()->isBitLogic() && portp->width() != 1) args += "&";  // it's a svBitVecVal
 
 		    args += portp->name()+"__Vcvt";
 
