@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -313,7 +313,7 @@ private:
 	if (!activityNumber) activityNumber++;   // For simplicity, always create it
 	int activityBits = VL_WORDS_I(activityNumber)*VL_WORDSIZE;   // For tighter code; round to next 32 bit point.
 	AstVar* newvarp = new AstVar (m_chgFuncp->fileline(), AstVarType::MODULETEMP,
-				      "__Vm_traceActivity", AstLogicPacked(), activityBits);
+				      "__Vm_traceActivity", AstBitPacked(), activityBits);
 	m_topModp->addStmtp(newvarp);
 	AstVarScope* newvscp = new AstVarScope(newvarp->fileline(), m_highScopep, newvarp);
 	m_highScopep->addVarp(newvscp);
@@ -328,8 +328,7 @@ private:
 		    vvertexp->insertp()->addNextHere
 			(new AstAssign (fl,
 					new AstSel (fl, new AstVarRef(fl, m_activityVscp, true),
-						    new AstConst(fl, acode),
-						    new AstConst(fl, 1)),
+						    acode, 1),
 					new AstConst (fl, AstConst::LogicTrue())));
 		}
 	    }
@@ -452,8 +451,7 @@ private:
 		    for (ActCodeSet::const_iterator csit = actset.begin(); csit!=actset.end(); ++csit) {
 			uint32_t acode = *csit;
 			AstNode* selp = new AstSel (fl, new AstVarRef(fl, m_activityVscp, false),
-						    new AstConst(fl, acode),
-						    new AstConst(fl, 1));
+						    acode, 1);
 			if (condp) condp = new AstOr (fl, condp, selp);
 			else condp = selp;
 		    }
