@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -56,7 +56,6 @@
 #include "V3Global.h"
 #include "V3Tristate.h"
 #include "V3Ast.h"
-#include "V3Const.h"
 #include "V3Stats.h"
 #include "V3Inst.h"
 
@@ -238,9 +237,10 @@ private:
 	enp->varType2Out();
 
 	if (enp->width() != enrhsp->width()) {
-	    if (enrhsp->width()==1) { // it seems from my futzing that the linter guarantees this condition
-		enrhsp = new AstReplicate(enrhsp->fileline(), enrhsp, new AstConst(enrhsp->fileline(), V3Number(enrhsp->fileline(), 32, enp->width())));
-		enrhsp->width(enp->width(), enp->widthMin());
+	    if (enrhsp->width1()) { // it seems from my futzing that the linter guarantees this condition
+		enrhsp = new AstReplicate(enrhsp->fileline(), enrhsp,
+					  new AstConst(enrhsp->fileline(), V3Number(enrhsp->fileline(), 32, enp->width())));
+		enrhsp->width(enp->width(), enp->width());  //minwidth==width
 	    } else {
 		enrhsp->v3error("Don't know how to deal with selection logic wider than 1 bit");
 	    }
@@ -440,7 +440,8 @@ private:
 			);
 
 		    selp->replaceWith(new AstVarRef(refp->fileline(), newlhsp, true));
-		    selp->deleteTree();
+		    pushDeletep(selp);  // Setting selp here or deleting immediately
+		    // breaks the t_tri_select test, this probably indicates a problem
 		} else {
 		    refp->varp(newlhsp); // assign the new var to the varref
 		    refp->name(newlhsp->name());
@@ -570,8 +571,8 @@ private:
 				      new AstVarRef(nodep->fileline(), enp, true));
 	    AstVarRef *rp = findVarRef(pinp);
 	    rp->replaceWith(new AstVarRef(nodep->fileline(), enp, true));
-	    rp->deleteTree();
-	    pinp->width(enp->width(),enp->widthMin());
+	    rp->deleteTree(); rp=NULL;
+	    pinp->width(enp->width(),enp->width());  // minwidth==width
 	    pinp->modVarp(enchildp);
 	    m_cellp->addPinsp(pinp);
 	    refp->user1p(enp);
@@ -590,6 +591,9 @@ private:
 public:
     // CONSTUCTORS
     TristateVisitor(AstNode* nodep) {
+	m_modp = NULL;
+	m_cellp = NULL;
+	m_unique = false;
 	nodep->accept(*this);
     }
     virtual ~TristateVisitor() { }
@@ -715,6 +719,7 @@ public:
 	m_modp = NULL;
 	m_cellp = NULL;
 	m_ftaskp = NULL;
+	m_state = CONVERT_VARS;
 	nodep->accept(*this);
     }
     virtual ~InoutVisitor() { }
