@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -385,8 +385,9 @@ private:
 	// anything lower will be renamed "uppernewname.lowerbegin"
 	bool lastBegin = m_inBegin;
 	m_inBegin = true;
-	nodep->iterateChildren(*this);
+	nodep->stmtsp()->iterateAndNext(*this);
 	m_inBegin = lastBegin;
+	nodep->flatsp()->iterateAndNext(*this);
 
 	if (m_varModeReplace && !m_inBegin // no upper begin, excluding this one
 	    ) {
