@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -85,15 +85,20 @@ enum Stage { PRELIM=1,FINAL=2,BOTH=3 };
 class WidthVP : public AstNUser {
     // Parameters to pass down hierarchy with visit functions.
     int	m_width;	// Expression width, for (2+2), it's 32 bits
-    int	m_minWidth;	// Minimum width, for (2+2), it's 2 bits, for 32'2+32'2 it's 32 bits
-    Stage	m_stage;	// If true, report errors
+    int	m_widthMin;	// Minimum width, for (2+2), it's 2 bits, for 32'2+32'2 it's 32 bits
+    Stage m_stage;	// If true, report errors
 public:
-    WidthVP(int width, int minWidth, Stage stage) : m_width(width), m_minWidth(minWidth), m_stage(stage) {}
+    WidthVP(int width, int widthMin, Stage stage) : m_width(width), m_widthMin(widthMin), m_stage(stage) {}
     int width() const { return m_width; }
-    int widthMin() const { return m_minWidth?m_minWidth:m_width; }
+    int widthMin() const { return m_widthMin?m_widthMin:m_width; }
     bool prelim() const { return m_stage&1; }
     bool final() const { return m_stage&2; }
+    char stageAscii() const { return "-PFB"[m_stage]; }
 };
+ostream& operator<<(ostream& str, const WidthVP* vup) {
+    str<<"  VUP(w="<<vup->width()<<",wm="<<vup->widthMin()<<",s="<<vup->stageAscii()<<")";
+    return str;
+}
 
 //######################################################################
 
@@ -136,7 +141,7 @@ private:
     virtual void visit(AstLogOr* nodep, AstNUser* vup) {	visit_log_O1_LR1rus(nodep,vup); }
     virtual void visit(AstLogIf* nodep, AstNUser* vup) {	visit_log_O1_LR1rus(nodep,vup); }  // Conversion from real not in IEEE, but a fallout
     virtual void visit(AstLogIff* nodep, AstNUser* vup) {	visit_log_O1_LR1rus(nodep,vup); }  // Conversion from real not in IEEE, but a fallout
- 
+
     // Widths: 1 bit out, Any width lhs
     virtual void visit(AstRedAnd* nodep, AstNUser* vup) {	visit_red_O1_Lrus(nodep,vup,false); }
     virtual void visit(AstRedOr* nodep, AstNUser* vup) {	visit_red_O1_Lrus(nodep,vup,false); }
@@ -145,7 +150,7 @@ private:
     virtual void visit(AstIsUnknown* nodep,AstNUser* vup) {	visit_red_O1_Lrus(nodep,vup,true); }  // Allow real
     virtual void visit(AstOneHot* nodep,AstNUser* vup) {	visit_red_O1_Lrus(nodep,vup,false); }
     virtual void visit(AstOneHot0* nodep,AstNUser* vup) {	visit_red_O1_Lrus(nodep,vup,false); }
- 
+
     // These have different node types, as they operate differently
     // Must add to case statement below,
     // Widths: 1 bit out, lhs width == rhs width.  real if lhs|rhs real
@@ -199,7 +204,7 @@ private:
     virtual void visit(AstNegate* nodep, AstNUser* vup) {	visit_math_Orus_Dreplace(nodep,vup,true); }
     // Unary never real
     virtual void visit(AstNot* nodep, AstNUser* vup) {		visit_math_Orus_Dreplace(nodep,vup,false); }
- 
+
     // Real: inputs and output real
     virtual void visit(AstAddD* nodep, AstNUser* vup) {		visit_math_Or_LRr(nodep,vup); }
     virtual void visit(AstSubD* nodep, AstNUser* vup) {		visit_math_Or_LRr(nodep,vup); }
@@ -215,18 +220,18 @@ private:
     virtual void visit(AstLogD* nodep, AstNUser* vup) {		visit_math_Or_Lr(nodep,vup); }
     virtual void visit(AstLog10D* nodep, AstNUser* vup) {	visit_math_Or_Lr(nodep,vup); }
     virtual void visit(AstSqrtD* nodep, AstNUser* vup) {	visit_math_Or_Lr(nodep,vup); }
- 
-    // Widths: out signed/unsigned width = lhs width, input un|signed 
+
+    // Widths: out signed/unsigned width = lhs width, input un|signed
     virtual void visit(AstSigned* nodep, AstNUser* vup) {	visit_Ous_Lus_Wforce(nodep,vup,AstNumeric::SIGNED); }
     virtual void visit(AstUnsigned* nodep, AstNUser* vup) {	visit_Ous_Lus_Wforce(nodep,vup,AstNumeric::UNSIGNED); }
- 
+
     // Widths: Output width from lhs, rhs<33 bits
     // Signed: If lhs signed
     virtual void visit(AstShiftL* nodep, AstNUser* vup) {	visit_shift_Ous_Lus_Rus32(nodep,vup); }
     virtual void visit(AstShiftR* nodep, AstNUser* vup) {	visit_shift_Ous_Lus_Rus32(nodep,vup); }
     // ShiftRS converts to ShiftR, but not vice-versa
     virtual void visit(AstShiftRS* nodep, AstNUser* vup) {	visit_shift_Ous_Lus_Rus32(nodep,vup); }
- 
+
     //========
     // Widths: Output real, input integer signed
     virtual void visit(AstBitsToRealD* nodep, AstNUser* vup) {	visit_Or_Lu64(nodep,vup); }
@@ -235,15 +240,15 @@ private:
     // Widths: Output integer signed, input real
     virtual void visit(AstRToIS* nodep, AstNUser* vup) {	visit_Os32_Lr(nodep,vup); }
     virtual void visit(AstRToIRoundS* nodep, AstNUser* vup) {	visit_Os32_Lr(nodep,vup); }
- 
+
     // Widths: Output integer unsigned, input real
     virtual void visit(AstRealToBits* nodep, AstNUser* vup) {	visit_Ou64_Lr(nodep,vup); }
 
     // Widths: Constant, terminal
-    virtual void visit(AstTime* nodep, AstNUser*) {		nodep->numeric(AstNumeric::UNSIGNED); nodep->width(64,64); }
-    virtual void visit(AstTimeD* nodep, AstNUser*) {		nodep->numeric(AstNumeric::DOUBLE); }
-    virtual void visit(AstTestPlusArgs* nodep, AstNUser*) {	nodep->numeric(AstNumeric::UNSIGNED); nodep->width(32,32); }
-    virtual void visit(AstScopeName* nodep, AstNUser* vup) {	nodep->width(64,1); }	// A pointer, but not that it matters
+    virtual void visit(AstTime* nodep, AstNUser*) {		nodep->dtypeChgUInt64(); }
+    virtual void visit(AstTimeD* nodep, AstNUser*) {		nodep->dtypeChgDouble(); }
+    virtual void visit(AstTestPlusArgs* nodep, AstNUser*) {	nodep->dtypeChgSigned32(); }
+    virtual void visit(AstScopeName* nodep, AstNUser*) {	nodep->dtypeChgUInt64(); }	// A pointer, but not that it matters
 
     // Special cases.  So many....
     virtual void visit(AstNodeCond* nodep, AstNUser* vup) {
@@ -266,7 +271,7 @@ private:
 	if (nodep->expr1p()->isDouble() || nodep->expr2p()->isDouble()) {
 	    spliceCvtD(nodep->expr1p());
 	    spliceCvtD(nodep->expr2p());
-	    nodep->numeric(AstNumeric::DOUBLE);
+	    nodep->dtypeChgDouble();
 	} else {
 	    int width  = max(vup->c()->width(),    max(nodep->expr1p()->width(),    nodep->expr2p()->width()));
 	    int mwidth = max(vup->c()->widthMin(), max(nodep->expr1p()->widthMin(), nodep->expr2p()->widthMin()));
@@ -280,7 +285,7 @@ private:
 	    nodep->expr1p()->iterateAndNext(*this,WidthVP(width,mwidth,FINAL).p());
 	    nodep->expr2p()->iterateAndNext(*this,WidthVP(width,mwidth,FINAL).p());
 	    // Error report and change sizes for suboperands of this node.
-	    widthCheckReduce(nodep,"Conditional Expression",nodep->condp(),1,0);
+	    widthCheckReduce(nodep,"Conditional Test",nodep->condp(),1,0);
 	    widthCheck(nodep,"Conditional True",nodep->expr1p(),width,mwidth);
 	    widthCheck(nodep,"Conditional False",nodep->expr2p(),width,mwidth);
 	}
@@ -295,7 +300,7 @@ private:
 	    checkCvtUS(nodep->rhsp());
 	    nodep->width(nodep->lhsp()->width() + nodep->rhsp()->width(),
 			 nodep->lhsp()->widthMin() + nodep->rhsp()->widthMin());
-	    nodep->numeric(AstNumeric::UNSIGNED); 
+	    nodep->numeric(AstNumeric::UNSIGNED);
 	    // Cleanup zero width Verilog2001 {x,{0{foo}}} now,
 	    // otherwise having width(0) will cause later assertions to fire
 	    if (AstReplicate* repp=nodep->lhsp()->castReplicate()) {
@@ -356,8 +361,7 @@ private:
 	    checkCvtUS(nodep->lsbp());
 	    int width = nodep->elementsConst();
 	    if (width > (1<<28)) nodep->v3error("Width of bit range is huge; vector of over 1billion bits: 0x"<<hex<<width);
-	    nodep->numeric(AstNumeric::UNSIGNED);
-	    nodep->width(width,width);
+	    // Note width() not set on range; use elementsConst()
 	    if (nodep->littleEndian()) {
 		nodep->v3warn(LITENDIAN,"Little bit endian vector: MSB < LSB of bit range: "<<nodep->lsbConst()<<":"<<nodep->msbConst());
 	    }
@@ -378,7 +382,7 @@ private:
 	    AstConst* widthConstp = nodep->widthp()->castConst();
 	    if (!widthConstp) {
 		nodep->v3error("Width of bit extract isn't a constant");
-		nodep->width(1,1); return;
+		nodep->dtypeChgLogicBool(); return;
 	    }
 	    int width = nodep->widthConst();
 	    nodep->width(width,width);
@@ -406,7 +410,7 @@ private:
 	    int frommsb = nodep->fromp()->width() - 1;
 	    int fromlsb = 0;
 	    AstNodeVarRef* varrp = nodep->fromp()->castNodeVarRef();
-	    if (varrp && varrp->varp()->basicp()->rangep()) {	// Selecting a bit from a multibit register
+	    if (varrp && varrp->varp()->basicp()->isRanged()) {	// Selecting a bit from a multibit register
 		frommsb = varrp->varp()->basicp()->msbMaxSelect();  // Corrected for negative lsb
 		fromlsb = varrp->varp()->basicp()->lsb();
 	    }
@@ -458,7 +462,7 @@ private:
 	    //
 	    int frommsb;
 	    int fromlsb;
-	    AstNodeDType* ddtypep = varrp->varp()->dtypeDimensionp(dimension);
+	    AstNodeDType* ddtypep = varrp->varp()->dtypep()->dtypeDimensionp(dimension);
 	    if (AstArrayDType* adtypep = ddtypep->castArrayDType()) {
 		int outwidth = varrp->width();		// Width of variable
 		frommsb = adtypep->msb();
@@ -527,7 +531,7 @@ private:
 	    if (nodep->num().sized()) {
 		nodep->width(nodep->num().width(), nodep->num().width());
 	    } else {
-		nodep->width(nodep->num().width(), nodep->num().minWidth());
+		nodep->width(nodep->num().width(), nodep->num().widthMin());
 	    }
 	}
 	// We don't size the constant until we commit the widths, as need parameters
@@ -538,8 +542,7 @@ private:
     }
     virtual void visit(AstRand* nodep, AstNUser* vup) {
 	if (vup->c()->prelim()) {
-	    nodep->numeric(AstNumeric::SIGNED);  // Says the spec
-	    nodep->width(32,32);  // Says the spec
+	    nodep->dtypeChgSigned32();  // Says the spec
 	}
     }
     virtual void visit(AstUCFunc* nodep, AstNUser* vup) {
@@ -559,8 +562,7 @@ private:
 	if (vup->c()->prelim()) {
 	    nodep->lhsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	    checkCvtUS(nodep->lhsp());
-	    nodep->numeric(AstNumeric::UNSIGNED);  // If want otherwise use a dpi import
-	    nodep->width(32,32);
+	    nodep->dtypeChgSigned32();
 	}
     }
     virtual void visit(AstPow* nodep, AstNUser* vup) {
@@ -616,23 +618,27 @@ private:
 	// But also cleanup array size
 	nodep->arrayp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	nodep->widthFrom(nodep->dtypep());
+	UINFO(4,"dtWidthed "<<nodep<<endl);
     }
     virtual void visit(AstBasicDType* nodep, AstNUser* vup) {
 	if (nodep->rangep()) {
 	    nodep->rangep()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
-	    nodep->widthFrom(nodep->rangep());
+	    nodep->width(nodep->rangep()->elementsConst(), nodep->rangep()->elementsConst());
 	}
 	// else width in node is correct; it was set based on keyword().width()
 	// at construction time.  Ditto signed, so "unsigned byte" etc works right.
+	UINFO(4,"dtWidthed "<<nodep<<endl);
     }
     virtual void visit(AstConstDType* nodep, AstNUser* vup) {
 	nodep->iterateChildren(*this, vup);
 	nodep->widthFrom(nodep->dtypep());
+	UINFO(4,"dtWidthed "<<nodep<<endl);
     }
     virtual void visit(AstRefDType* nodep, AstNUser* vup) {
 	nodep->iterateChildren(*this, vup);
 	if (nodep->defp()) nodep->defp()->iterate(*this,vup);
 	nodep->widthSignedFrom(nodep->dtypeSkipRefp());
+	UINFO(4,"dtWidthed "<<nodep<<endl);
     }
     virtual void visit(AstTypedef* nodep, AstNUser* vup) {
 	nodep->iterateChildren(*this, vup);
@@ -694,16 +700,22 @@ private:
 	AstBasicDType* bdtypep = nodep->dtypep()->castBasicDType();
 	bool implicitParam = nodep->isParam() && bdtypep && bdtypep->implicit();
 	if (implicitParam) {
-	    AstNumeric rs = AstNumeric::UNSIGNED;
 	    int width=0;
+	    AstNumeric rs = AstNumeric::UNSIGNED;
 	    if (nodep->valuep()) {
 		nodep->valuep()->iterateAndNext(*this,WidthVP(width,0,PRELIM).p());
+		UINFO(9,"implicitParamPRELIMIV "<<nodep->valuep()<<endl);
 		// Although nodep will get a different width for parameters just below,
 		// we want the init numbers to retain their width/minwidth until parameters are replaced.
 		// This prevents width warnings at the location the parameter is substituted in
-		nodep->valuep()->iterateAndNext(*this,WidthVP(width,0,FINAL).p());
-		rs = nodep->numeric();
-		if (!rs.isDouble()) {
+		if (nodep->valuep()->isDouble()) {
+		    nodep->dtypeChgDouble(); bdtypep=NULL;
+		    nodep->dtypep()->dtypeChgDouble(); bdtypep=NULL;
+		    nodep->valuep()->iterateAndNext(*this,WidthVP(width,0,FINAL).p());
+		} else {
+		    nodep->valuep()->iterateAndNext(*this,WidthVP(width,0,FINAL).p());
+		    if (bdtypep->nosigned()) rs = nodep->valuep()->numeric();
+		    else rs = nodep->numeric();
 		    if (nodep->valuep()->widthSized()) {
 			width = nodep->valuep()->width();
 		    } else {
@@ -711,24 +723,20 @@ private:
 			width = 32;
 		    }
 		}
-	    }
-	    // Parameter sizes can come from the thing they get assigned from
-	    // They then "stick" to that width.
-	    if (!width) width=32;	// Or, if nothing, they're 32 bits.
-	    // CLEANUP: If we have a TypeOf operator, or make dtype on every AstNode,
-	    // this would be just a copy of the dtype into the var.
-	    if (rs.isDouble()) {
-		AstBasicDType* newp = new AstBasicDType(nodep->fileline(), AstBasicDTypeKwd::DOUBLE);
-		bdtypep->replaceWith(newp);
-		bdtypep->deleteTree(); bdtypep=NULL;
+		UINFO(9,"implicitParamFromIV "<<nodep->valuep()<<endl);
+		//UINFO below will print variable nodep
 	    } else {
+		// Or, if nothing assigned, they're 32 bits.
+		width=32;
+	    }
+	    if (!nodep->isDouble()) {
 		AstBasicDType* newp = new AstBasicDType(nodep->fileline(), AstLogicPacked(), width);
 		newp->implicit(true);
 		newp->numeric(rs);  // SIGNED or UNSIGNED
 		bdtypep->replaceWith(newp);
 		bdtypep->deleteTree(); bdtypep=NULL;
+		nodep->dtypep()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	    }
-	    nodep->dtypep()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	}
 	else { // non param or sized param
 	    nodep->dtypep()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
@@ -740,7 +748,7 @@ private:
 	// See above note about valuep()->...FINAL
 	nodep->widthSignedFrom(nodep->dtypep());
 	if (nodep->valuep()) {
-	    if (implicitParam) {
+	    if (implicitParam && !nodep->isDouble()) {
 		nodep->width(nodep->width(), nodep->valuep()->widthMin());   // Needed as mwidth might not equal width
 	    }
 	    widthCheck(nodep,"Initial value",nodep->valuep(),
@@ -817,7 +825,7 @@ private:
 	    for (; enump; enump=enump->backp()) {
 		if (enump->castEnumDType()) break;
 	    }
-	    if (!enump) nodep->v3fatalSrc("EnumItemRef can't deref back to a Enum");
+	    if (!enump) nodep->v3fatalSrc("EnumItemRef can't deref back to an Enum");
 	    enump->iterate(*this,vup);
 	}
 	nodep->widthSignedFrom(nodep->itemp());
@@ -830,8 +838,7 @@ private:
 	    widthCheckReduce(nodep,"Disable",nodep->disablep(),1,1); // it's like an if() condition.
 	}
 	widthCheckReduce(nodep,"Property",nodep->propp(),1,1);	// it's like an if() condition.
-	nodep->numeric(AstNumeric::UNSIGNED);
-	nodep->width(1,1);
+	nodep->dtypeChgLogicBool();
     }
 
     //--------------------
@@ -842,47 +849,34 @@ private:
 	AstNodeCase* lastCasep = m_casep;
 	m_casep = nodep;
 	nodep->exprp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,PRELIM).p());
-	for (AstCaseItem* nextp, *itemp = nodep->itemsp(); itemp; itemp=nextp) {
-	    nextp = itemp->nextp()->castCaseItem(); // Prelim may cause the node to get replaced
-	    itemp->iterate(*this,WidthVP(ANYSIZE,0,PRELIM).p());  itemp=NULL;
+	for (AstCaseItem* nextip, *itemp = nodep->itemsp(); itemp; itemp=nextip) {
+	    nextip = itemp->nextp()->castCaseItem(); // Prelim may cause the node to get replaced
+	    if (!m_casep->castGenCase()) itemp->bodysp()->iterateAndNext(*this);
+	    for (AstNode* nextcp, *condp = itemp->condsp(); condp; condp=nextcp) {
+		nextcp = condp->nextp(); // Prelim may cause the node to get replaced
+		condp->iterate(*this,WidthVP(ANYSIZE,0,PRELIM).p()); condp=NULL;
+	    }
 	}
+	// Take width as maximum across all items
 	int width = nodep->exprp()->width();
 	int mwidth = nodep->exprp()->widthMin();
 	for (AstCaseItem* itemp = nodep->itemsp(); itemp; itemp=itemp->nextp()->castCaseItem()) {
-	    width = max(width,itemp->width());
-	    mwidth = max(mwidth,itemp->widthMin());
+	    for (AstNode* condp = itemp->condsp(); condp; condp=condp->nextp()) {
+		width = max(width,condp->width());
+		mwidth = max(mwidth,condp->widthMin());
+	    }
 	}
+	// Apply width
 	nodep->exprp()->iterateAndNext(*this,WidthVP(width,mwidth,FINAL).p());
 	for (AstCaseItem* itemp = nodep->itemsp(); itemp; itemp=itemp->nextp()->castCaseItem()) {
-	    itemp->iterate(*this,WidthVP(width,mwidth,FINAL).p());
+	    for (AstNode* condp = itemp->condsp(); condp; condp=condp->nextp()) {
+		condp->iterate(*this,WidthVP(width,mwidth,FINAL).p());
+		widthCheck(nodep,"Case Item",condp,width,mwidth);
+	    }
 	}
         widthCheck(nodep,"Case expression",nodep->exprp(),width,mwidth);
 	m_casep = lastCasep;
     }
-    virtual void visit(AstCaseItem* nodep, AstNUser* vup) {
-	// Same for both prelim() and final()
-	if (!m_casep->castGenCase()) nodep->bodysp()->iterateAndNext(*this);
-	if (!nodep->condsp()) {
-	    // Else "default:" of the case, just return benign value
-	    nodep->width(vup->c()->width(),vup->c()->widthMin());
-	} else {
-	    // Need to look across multiple case values for one set of statements
-	    int width = nodep->condsp()->width();
-	    int mwidth = nodep->condsp()->widthMin();
-	    for (AstNode* nextp, *condp = nodep->condsp(); condp; condp=nextp) {
-		nextp = condp->nextp(); // Prelim may cause the node to get replaced
-		condp->iterate(*this,vup); condp=NULL;
-	    }
-	    for (AstNode* condp = nodep->condsp(); condp; condp=condp->nextp()) {
-		width = max(width,condp->width());
-		mwidth = max(mwidth,condp->widthMin());
-		if (vup->c()->final()) {
-		    widthCheck(nodep,"Case Item",condp,vup->c()->width(),vup->c()->widthMin());
-		}
-	    }
-	    nodep->width(width,mwidth);
-	}
-    }
     virtual void visit(AstNodeFor* nodep, AstNUser*) {
 	// TOP LEVEL NODE
 	nodep->initsp()->iterateAndNext(*this);
@@ -1019,7 +1013,7 @@ private:
 	nodep->filep()->iterateAndNext(*this,WidthVP(32,32,BOTH).p());
 	nodep->strgp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	if (vup->c()->prelim()) {
-	    nodep->width(32,32);
+	    nodep->dtypeChgSigned32();  // Spec says integer return
 	}
 	widthCheck(nodep,"file_descriptor",nodep->filep(),32,32);
     }
@@ -1027,7 +1021,7 @@ private:
 	nodep->filep()->iterateAndNext(*this,WidthVP(32,32,BOTH).p());
 	nodep->exprsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	if (vup->c()->prelim()) {
-	    nodep->width(32,32);
+	    nodep->dtypeChgSigned32();  // Spec says integer return
 	}
 	widthCheck(nodep,"file_descriptor",nodep->filep(),32,32);
     }
@@ -1035,7 +1029,7 @@ private:
 	nodep->fromp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	nodep->exprsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	if (vup->c()->prelim()) {
-	    nodep->width(32,32);
+	    nodep->dtypeChgSigned32();  // Spec says integer return
 	}
     }
     virtual void visit(AstSysIgnore* nodep, AstNUser* vup) {
@@ -1043,8 +1037,7 @@ private:
     }
     virtual void visit(AstSystemF* nodep, AstNUser*) {
 	nodep->lhsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
-	nodep->numeric(AstNumeric::UNSIGNED);
-	nodep->width(32,32);
+	nodep->dtypeChgSigned32();  // Spec says integer return
     }
     virtual void visit(AstSystemT* nodep, AstNUser*) {
 	nodep->lhsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
@@ -1057,8 +1050,7 @@ private:
     }
     virtual void visit(AstValuePlusArgs* nodep, AstNUser* vup) {
 	nodep->exprsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
-	nodep->numeric(AstNumeric::UNSIGNED);
-	nodep->width(32,32);
+	nodep->dtypeChgSigned32();  // Spec says integer return
     }
     virtual void visit(AstUCStmt* nodep, AstNUser*) {
 	// TOP LEVEL NODE
@@ -1102,7 +1094,7 @@ private:
 	    bool inputPin = nodep->modVarp()->isInput();
 	    int awidth;
 	    if (m_cellRangep) {
-		int numInsts = m_cellRangep->width();
+		int numInsts = m_cellRangep->elementsConst();
 		if (expwidth == pinwidth) {
 		    awidth = pinwidth;	// Arrayed instants: widths match so connect to each instance
 		} else if (expwidth == pinwidth*numInsts) {
@@ -1167,7 +1159,7 @@ private:
 	UINFO(5,"  FTASK "<<nodep<<endl);
 	if (nodep->doingWidth()) {
 	    nodep->v3error("Unsupported: Recursive function or task call");
-	    nodep->width(1,1);
+	    nodep->dtypeChgLogicBool();
 	    nodep->didWidth(true);
 	    return;
 	}
@@ -1261,9 +1253,9 @@ private:
 			if ((portp->isOutput() || portp->isInout())
 			    && pinp->width() != portp->width()) {
 			    pinp->v3error("Unsupported: Function output argument '"<<portp->prettyName()<<"'"
-					  <<" requires "<<pinp->width()
+					  <<" requires "<<portp->width()
 					  <<" bits, but connection's "<<pinp->prettyTypeName()
-					  <<" generates "<<portp->width()<<" bits.");
+					  <<" generates "<<pinp->width()<<" bits.");
 			    // otherwise would need some mess to force both sides to proper size
 			    // (get an ASSIGN with EXTEND on the lhs instead of rhs)
 			}
@@ -1297,43 +1289,46 @@ private:
     // WIDTH METHODs -- all iterate
 
     void visit_Or_Lu64(AstNodeUniop* nodep, AstNUser* vup) {
+	// CALLER: AstBitsToRealD
 	// Real: Output real
 	if (vup->c()->prelim()) {  // First stage evaluation
 	    nodep->lhsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	    checkCvtUS(nodep->lhsp());
-	    nodep->numeric(AstNumeric::DOUBLE);
+	    nodep->dtypeChgDouble();
 	    widthCheck(nodep,"LHS",nodep->lhsp(),64,64);
 	}
     }
     void visit_Or_Ls32(AstNodeUniop* nodep, AstNUser* vup) {
+	// CALLER: AstIToRD
 	// Real: Output real
 	if (vup->c()->prelim()) {  // First stage evaluation
 	    nodep->lhsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	    checkCvtUS(nodep->lhsp());
-	    nodep->numeric(AstNumeric::DOUBLE);
+	    nodep->dtypeChgDouble();
 	    widthCheck(nodep,"LHS",nodep->lhsp(),32,32);
 	}
     }
     void visit_Os32_Lr(AstNodeUniop* nodep, AstNUser* vup) {
+	// CALLER: RToI
 	// Real: LHS real
 	if (vup->c()->prelim()) {  // First stage evaluation
 	    nodep->lhsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	    checkCvtD(nodep->lhsp());
-	    nodep->numeric(AstNumeric::SIGNED);
-	    nodep->width(32,32);
+	    nodep->dtypeChgSigned32();
 	}
     }
     void visit_Ou64_Lr(AstNodeUniop* nodep, AstNUser* vup) {
+	// CALLER: RealToBits
 	// Real: LHS real
 	if (vup->c()->prelim()) {  // First stage evaluation
 	    nodep->lhsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	    checkCvtD(nodep->lhsp());
-	    nodep->numeric(AstNumeric::UNSIGNED);
-	    nodep->width(64,64);
+	    nodep->dtypeChgUInt64();
 	}
     }
 
     void visit_log_O1_L1rus(AstNode* nodep, AstNUser* vup) {
+	// CALLER: LogNot, PslBool
 	// Note AstPslBool isn't a AstNodeUniop, or we'd only allow that here
 	// Widths: 1 bit out, lhs 1 bit
 	// Real: Allowed; implicitly compares with zero
@@ -1345,13 +1340,13 @@ private:
 	    nodep->op1p()->iterateAndNext(*this,WidthVP(1,0,BOTH).p());
 	    spliceCvtCmpD0(nodep->op1p());
 	}
-	nodep->width(1,1);
-	nodep->numeric(AstNumeric::UNSIGNED);
+	nodep->dtypeChgLogicBool();
 	if (vup->c()->final()) {
 	    widthCheckReduce(nodep,"LHS",nodep->op1p(),1,1);
 	}
     }
     void visit_log_O1_LR1rus(AstNodeBiop* nodep, AstNUser* vup) {
+	// CALLER: LogAnd, LogOr, LogIf, LogIff
 	// Widths: 1 bit out, lhs 1 bit, rhs 1 bit
 	if (vup->c()->prelim()) {
 	    nodep->lhsp()->iterateAndNext(*this,WidthVP(1,0,BOTH).p());
@@ -1359,8 +1354,7 @@ private:
 	    spliceCvtCmpD0(nodep->lhsp());
 	    spliceCvtCmpD0(nodep->rhsp());
 	}
-	nodep->width(1,1);
-	nodep->numeric(AstNumeric::UNSIGNED);
+	nodep->dtypeChgLogicBool();
 	if (vup->c()->final()) {
 	    widthCheckReduce(nodep,"LHS",nodep->lhsp(),1,1);
 	    widthCheckReduce(nodep,"RHS",nodep->rhsp(),1,1);
@@ -1368,20 +1362,21 @@ private:
     }
 
     void visit_red_O1_Lrus(AstNodeUniop* nodep, AstNUser* vup, bool realok) {
+	// CALLER: RedAnd, RedOr, ..., IsUnknown
 	// Widths: 1 bit out, Any width lhs
 	// Signed: Output unsigned, Lhs/Rhs/etc non-real
 	if (vup->c()->prelim()) {
 	    nodep->lhsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	}
 	if (!realok) checkCvtUS(nodep->lhsp());
-	nodep->numeric(AstNumeric::UNSIGNED);
-	nodep->width(1,1);
+	nodep->dtypeChgLogicBool();
     }
     void visit_cmp_O1_DSreplace(AstNodeBiop* nodep, AstNUser* vup) {
+	// CALLER: AstEq, AstGt, ..., AstLtS
 	// COMPARES
 	// Widths: 1 bit out, lhs width == rhs width
 	// Signed: if RHS&LHS signed, OPERATOR CHANGES to signed flavor
-	// Real: allowed on RHS, if RHS|LHS is real, both become real, and OPERATOR CHANGES 
+	// Real: allowed on RHS, if RHS|LHS is real, both become real, and OPERATOR CHANGES
 	if (vup->c()->prelim()) {
 	    nodep->lhsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,PRELIM).p());
 	    nodep->rhsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,PRELIM).p());
@@ -1400,8 +1395,7 @@ private:
 	}
 	int width  = max(nodep->lhsp()->width(),    nodep->rhsp()->width());
 	int ewidth = max(nodep->lhsp()->widthMin(), nodep->rhsp()->widthMin());
-	nodep->numeric(AstNumeric::UNSIGNED);
-	nodep->width(1,1);
+	nodep->dtypeChgLogicBool();
 	if (vup->c()->final()) {
 	    nodep->lhsp()->iterateAndNext(*this,WidthVP(width,ewidth,FINAL).p());
 	    nodep->rhsp()->iterateAndNext(*this,WidthVP(width,ewidth,FINAL).p());
@@ -1410,6 +1404,8 @@ private:
 	}
     }
     void visit_cmp_O1_LRrus(AstNodeBiop* nodep, AstNUser* vup, bool real_lhs) {
+	// CALLER: (real_lhs=true) EqD, LtD
+	// CALLER: (real_lhs=false) EqCase, NeqCase
 	// Widths: 1 bit out, lhs width == rhs width
 	// Signed doesn't matter
 	// Real if and only if real_lhs set
@@ -1427,8 +1423,7 @@ private:
 	}
 	int width  = max(nodep->lhsp()->width(),    nodep->rhsp()->width());
 	int ewidth = max(nodep->lhsp()->widthMin(), nodep->rhsp()->widthMin());
-	nodep->numeric(AstNumeric::UNSIGNED);
-	nodep->width(1,1);
+	nodep->dtypeChgLogicBool();
 	if (vup->c()->final()) {
 	    nodep->lhsp()->iterateAndNext(*this,WidthVP(width,ewidth,FINAL).p());
 	    nodep->rhsp()->iterateAndNext(*this,WidthVP(width,ewidth,FINAL).p());
@@ -1438,6 +1433,8 @@ private:
     }
 
     void visit_math_Orus_Dreplace(AstNodeUniop* nodep, AstNUser* vup, bool real_ok) {
+	// CALLER: (real_ok=false) Not
+	// CALLER: (real_ok=true) Negate
 	// Widths: out width = lhs width
 	// Signed: From lhs
 	// "Interim results shall take the max of operands, including LHS of assignments"
@@ -1467,6 +1464,7 @@ private:
     }
 
     void visit_Ous_Lus_Wforce(AstNodeUniop* nodep, AstNUser* vup, AstNumeric rs_out) {
+	// CALLER: Signed, Unsigned
 	// Widths: out width = lhs width
 	// It always comes exactly from LHS; ignores any upper operand
 	if (nodep->op2p()) nodep->v3fatalSrc("For unary ops only!");
@@ -1475,7 +1473,7 @@ private:
 	    checkCvtUS(nodep->lhsp());
 	}
 	int width  = nodep->lhsp()->width();
-	int ewidth = nodep->lhsp()->width();  // Not minWidth; force it.
+	int ewidth = nodep->lhsp()->width();  // Not widthMin; force it.
 	nodep->width(width,ewidth);
 	nodep->numeric(rs_out);
 	if (vup->c()->final()) {
@@ -1486,6 +1484,7 @@ private:
     }
 
     void visit_shift_Ous_Lus_Rus32(AstNodeBiop* nodep, AstNUser* vup)  {
+	// CALLER: ShiftL, ShiftR, ShiftRS
 	// Widths: Output width from lhs, rhs<33 bits
 	// Signed: Output signed iff LHS signed; unary operator
 	shift_prelim(nodep,vup);
@@ -1517,13 +1516,14 @@ private:
 	    nodep->lhsp()->iterateAndNext(*this,WidthVP(width,ewidth,FINAL).p());
 	    widthCheck(nodep,"LHS",nodep->lhsp(),width,ewidth);
 	    if (nodep->rhsp()->width()>32)
-		nodep->rhsp()->v3error("Unsupported: Shifting of by a over 32 bit number isn't supported."
+		nodep->rhsp()->v3error("Unsupported: Shifting of by over 32-bit number isn't supported."
 				       <<" (This isn't a shift of 32 bits, but a shift of 2^32, or 4 billion!)\n");
 	}
 	return nodep;  // May edit
     }
 
     void visit_boolmath_Ous_LRus(AstNodeBiop* nodep, AstNUser* vup) {
+	// CALLER: And, Or, Xor, ...
 	// Widths: out width = lhs width = rhs width
 	// Signed: if lhs & rhs signed
 	if (!nodep->rhsp()) nodep->v3fatalSrc("For binary ops only!");
@@ -1561,6 +1561,8 @@ private:
     }
 
     void visit_math_Orus_DSreplace(AstNodeBiop* nodep, AstNUser* vup, bool real_ok) {
+	// CALLER: (real_ok=false) AddS, SubS, ...
+	// CALLER: (real_ok=true) Add, Sub, ...
 	// Widths: out width = lhs width = rhs width
 	// Signed: Replace operator with signed operator, or signed to unsigned
 	// Real: Replace operator with real operator
@@ -1611,19 +1613,21 @@ private:
 	}
     }
     void visit_math_Or_LRr(AstNodeBiop* nodep, AstNUser* vup) {
+	// CALLER: AddD, MulD, ...
 	if (vup->c()->prelim()) {  // First stage evaluation
 	    nodep->lhsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	    nodep->rhsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	    checkCvtD(nodep->lhsp());
 	    checkCvtD(nodep->rhsp());
-	    nodep->numeric(AstNumeric::DOUBLE);
+	    nodep->dtypeChgDouble();
 	}
     }
     void visit_math_Or_Lr(AstNodeUniop* nodep, AstNUser* vup) {
+	// CALLER: Negate, Ceil, Log, ...
 	if (vup->c()->prelim()) {  // First stage evaluation
 	    nodep->lhsp()->iterateAndNext(*this,WidthVP(ANYSIZE,0,BOTH).p());
 	    checkCvtD(nodep->lhsp());
-	    nodep->numeric(AstNumeric::DOUBLE);
+	    nodep->dtypeChgDouble();
 	}
     }
 
