@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -78,12 +78,12 @@ private:
 	// Perform error checks on the node
 	AstVar* varp = varFromBasefrom(basefromp);
 	//UINFO(9,"SCD\n"); if (debug()>=9) nodep->backp()->dumpTree(cout,"-selcheck: ");
-	AstNodeDType* ddtypep = varp->dtypeDimensionp(dimension);
+	AstNodeDType* ddtypep = varp->dtypep()->dtypeDimensionp(dimension);
 	if (AstArrayDType* adtypep = ddtypep->castArrayDType()) {
 	    return adtypep;
 	}
 	else if (AstBasicDType* adtypep = ddtypep->castBasicDType()) {
-	    if (!adtypep->rangep()) {
+	    if (!adtypep->isRanged()) {
 		nodep->v3error("Illegal bit select; variable does not have a bit range, or bad dimension: "<<varp->prettyName());
 		return NULL;
 	    }
@@ -108,14 +108,15 @@ private:
 	    AstNode* newp = new AstConst(lhsp->fileline(), num);
 	    return newp;
 	} else if (rhs > 0) {
-	    // We must make sure sub gets sign of original value
 	    AstNode* newp = new AstSub(lhsp->fileline(), lhsp,
 				       new AstConst(lhsp->fileline(), AstConst::Unsized32(), rhs));
+	    // We must make sure sub gets sign of original value, not from the constant
 	    newp->numericFrom(lhsp);
 	    return newp;
 	} else {  // rhs < 0;
 	    AstNode* newp = new AstAdd(lhsp->fileline(), lhsp,
 				       new AstConst(lhsp->fileline(), AstConst::Unsized32(), -rhs));
+	    // We must make sure sub gets sign of original value, not from the constant
 	    newp->numericFrom(lhsp);
 	    return newp;
 	}
@@ -126,7 +127,7 @@ private:
 	AstNode* newp = new AstSub(rhsp->fileline(),
 				   new AstConst(rhsp->fileline(), AstConst::Unsized32(), lhs),
 				   rhsp);
-	newp->numericFrom(rhsp);  // Important as AstSub default is lhs's sign
+	newp->widthSignedFrom(rhsp);  // Important as AstSub default is lhs's sign
 	return newp;
     }
 
@@ -143,14 +144,15 @@ private:
 	// Don't report WIDTH warnings etc here, as may be inside a generate branch that will be deleted
 	AstVar* varp = varFromBasefrom(basefromp);
 	// SUB #'s Not needed when LSB==0 and MSB>=0 (ie [0:-13] must still get added!)
-	if (!varp->basicp()->rangep()) {
-	    // vector without range is ok, for example a INTEGER x; y = x[21:0];
+	if (!varp->basicp()->isRanged()) {
+	    // vector without range, or 0 lsb is ok, for example a INTEGER x; y = x[21:0];
 	    return underp;
 	} else {
-	    if (!varp->basicp()->rangep()->msbp()->castConst()
-		|| !varp->basicp()->rangep()->lsbp()->castConst())
+	    if (varp->basicp()->rangep()
+		&& (!varp->basicp()->rangep()->msbp()->castConst()
+		    || !varp->basicp()->rangep()->lsbp()->castConst()))
 		varp->v3fatalSrc("Non-constant variable range; errored earlier");  // in constifyParam(varp)
-	    if (varp->basicp()->rangep()->littleEndian()) {
+	    if (varp->basicp()->littleEndian()) {
 		// reg [1:3] was swapped to [3:1] (lsbEndianedp==3) and needs a SUB(3,under)
 		AstNode* newp = newSubNeg(varp->basicp()->msb(), underp);
 		return newp;
@@ -262,7 +264,7 @@ private:
 	    }
 	} else if (AstBasicDType* adtypep = ddtypep->castBasicDType()) {
 	    if (adtypep) {} // Unused
-	    if (varp->basicp()->rangep() && varp->basicp()->rangep()->littleEndian()) {
+	    if (varp->basicp()->littleEndian()) {
 		// Below code assumes big bit endian; just works out if we swap
 		int x = msb; msb = lsb; lsb = x;
 	    }
@@ -307,7 +309,7 @@ private:
 	if (AstBasicDType* adtypep = ddtypep->castBasicDType()) {
 	    AstSel* newp = NULL;
 	    if (nodep->castSelPlus()) {
-		if (adtypep->rangep() && adtypep->rangep()->littleEndian()) {
+		if (adtypep->littleEndian()) {
 		    // SELPLUS(from,lsb,width) -> SEL(from, (vector_msb-width+1)-sel, width)
 		    newp = new AstSel (nodep->fileline(),
 				       fromp,
@@ -321,7 +323,7 @@ private:
 				       widthp);
 		}
 	    } else if (nodep->castSelMinus()) {
-		if (adtypep->rangep() && adtypep->rangep()->littleEndian()) {
+		if (adtypep->littleEndian()) {
 		    // SELMINUS(from,msb,width) -> SEL(from, msb-[bit])
 		    newp = new AstSel (nodep->fileline(),
 				       fromp,
