@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2011 by Wilson Snyder.  This program is free software; you can
+// Copyright 2003-2012 by Wilson Snyder.  This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -94,6 +94,17 @@ V3Global v3Global;
 //######################################################################
 // V3 Class -- top level
 
+AstNetlist* V3Global::makeNetlist() {
+    AstNetlist* newp = new AstNetlist();
+    return newp;
+}
+
+void V3Global::checkTree() { rootp()->checkTree(); }
+
+void V3Global::clear() {
+    if (m_rootp) m_rootp->deleteTree(); m_rootp=NULL;
+}
+
 void V3Global::readFiles() {
     // NODE STATE
     //   AstNode::user4p()	// V3SymTable*    Package and typedef symbol names
@@ -173,7 +184,8 @@ void process () {
 
     // Commit to the widths we've chosen; Make widthMin==width
     V3Width::widthCommit(v3Global.rootp());
-    v3Global.assertWidthsSame(true);
+    v3Global.assertDTypesResolved(true);
+    v3Global.assertWidthsMatch(true);
     //v3Global.rootp()->dumpTreeFile(v3Global.debugFilename("widthcommit.tree"));
 
     // Coverage insertion
@@ -478,9 +490,9 @@ void process () {
 
     // Here down, widthMin() is the Verilog width, and width() is the C++ width
     // Bits between widthMin() and width() are irrelevant, but may be non zero.
-    v3Global.assertWidthsSame(false);
+    v3Global.assertWidthsMatch(false);
 
-    // Make all operations a multiple of 32 bits
+    // Make all math operations either 8, 16, 32 or 64 bits
     V3Clean::cleanAll(v3Global.rootp());
     v3Global.rootp()->dumpTreeFile(v3Global.debugFilename("clean.tree"));
 
@@ -578,6 +590,8 @@ int main(int argc, char** argv, char** env) {
     // Check environment
     V3Options::getenvSYSTEMC();
     V3Options::getenvSYSTEMC_ARCH();
+    V3Options::getenvSYSTEMC_INCLUDE();
+    V3Options::getenvSYSTEMC_LIBDIR();
     V3Options::getenvSYSTEMPERL();
     V3Options::getenvSYSTEMPERL_INCLUDE();
 
