@@ -48,6 +48,7 @@ my @opt_tests;
 my $opt_gdb;
 my $opt_gdbbt;
 my $opt_gdbsim;
+my $opt_ghdl;
 my $opt_iv;
 my $opt_jobs = 1;
 my $opt_nc;
@@ -70,6 +71,7 @@ if (! GetOptions (
 		  "gdb!"	=> \$opt_gdb,
 		  "gdbbt!"	=> \$opt_gdbbt,
 		  "gdbsim!"	=> \$opt_gdbsim,
+		  "ghdl!"	=> \$opt_ghdl,
 		  "golden!"	=> sub { $ENV{HARNESS_UPDATE_GOLDEN} = 1; },
 		  "help"	=> \&usage,
 		  "iverilog!"	=> \$opt_iv,
@@ -95,7 +97,7 @@ $opt_jobs = calc_jobs() if defined $opt_jobs && $opt_jobs==0;
 
 $Fork->max_proc($opt_jobs);
 
-if (!$opt_atsim && !$opt_iv && !$opt_vcs && !$opt_nc && !$opt_vlt) {
+if (!$opt_atsim && !$opt_ghdl && !$opt_iv && !$opt_vcs && !$opt_nc && !$opt_vlt) {
     $opt_vlt = 1;
 }
 
@@ -120,6 +122,7 @@ my @fails;
 
 foreach my $testpl (@opt_tests) {
     one_test(pl_filename => $testpl, atsim=>1) if $opt_atsim;
+    one_test(pl_filename => $testpl, ghdl=>1) if $opt_ghdl;
     one_test(pl_filename => $testpl, iv=>1) if $opt_iv;
     one_test(pl_filename => $testpl, nc=>1) if $opt_nc;
     one_test(pl_filename => $testpl, vcs=>1) if $opt_vcs;
@@ -130,10 +133,13 @@ $Fork->wait_all();   # Wait for all children to finish
 
 sub one_test {
     my @params = @_;
+    my %params = (@params);
     $leftcnt++;
     $Fork->schedule
 	(
+	 test_pl_filename => $params{pl_filename},
 	 run_on_start => sub {
+	     # Running in context of child, so can't pass data to parent directly
 	     print ("="x70,"\n");
 	     my $test = VTest->new(@params);
 	     $test->oprint("="x50,"\n");
@@ -165,6 +171,11 @@ sub one_test {
 		     .$test->{pl_filename}." ".join(' ',@Orig_ARGV_Sw)."\n";
 		 $failcnt++;
 		 report(\@fails, $Log_Filename);
+		 my $other = "";
+		 foreach my $proc ($Fork->running) {
+		     $other .= "  ".$proc->{test_pl_filename};
+		 }
+		 $test->oprint("Simultaneous running tests:",$other,"\n") if $other;
 		 if ($opt_stop) { die "%Error: --stop and errors found\n"; }
 	     }
 	     $leftcnt--;
@@ -267,7 +278,19 @@ sub new {
     my $self = {@_};
 
     $self->{name} ||= $2 if $self->{pl_filename} =~ m!^(.*/)?([^/]*)\.pl$!;
-    $self->{obj_dir} ||= "obj_dir/$self->{name}";
+
+    $self->{mode} = "";
+    $self->{mode} ||= "atsim" if $self->{atsim};
+    $self->{mode} ||= "ghdl" if $self->{ghdl};
+    $self->{mode} ||= "vcs" if $self->{vcs};
+    $self->{mode} ||= "vlt" if $self->{vlt};
+    $self->{mode} ||= "nc" if $self->{nc};
+    $self->{mode} ||= "iv" if $self->{iv};
+
+    # For backward compatibility, the verilator tests have no prefix
+    $self->{obj_dir} ||= ("obj_dir/".($self->{mode} eq 'vlt' ? "" : $self->{mode}."_")
+			  ."$self->{name}");
+
     foreach my $dir (@::Test_Dirs) {
 	# t_dir used both absolutely and under obj_dir
 	if (-e "$dir/$self->{name}.pl") {
@@ -280,9 +303,9 @@ sub new {
 	}
     }
     $self->{t_dir} or die "%Error: Can't locate dir for $self->{name},";
+
     $self = {
 	name => undef, 		# Set below, name of this test
-	mode => "",
 	pl_filename => undef,	# Name of .pl file to get setup from
 	make_top_shell => 1,	# Make a default __top.v file
 	make_main => 1,		# Make __main.cpp
@@ -302,25 +325,32 @@ sub new {
 	all_run_flags => [],
         # ATSIM
 	atsim => 0,
-	atsim_flags => [split(/\s+/,"-c +sv +define+atsim"),
+	atsim_flags => [split(/\s+/,"-c +sv +define+ATSIM"),
 			"+sv_dir+$self->{obj_dir}/.athdl_compile"],
 	atsim_flags2 => [],  # Overridden in some sim files
-	atsimrun_flags => [],
+	atsim_run_flags => [],
+        # GHDL
+	ghdl => 0,
+	ghdl_work_dir => "$self->{obj_dir}/ghdl_compile",
+	ghdl_flags => [($::Debug?"-v":""),
+		       "--workdir=$self->{obj_dir}/ghdl_compile", ],
+	ghdl_flags2 => [],  # Overridden in some sim files
+	ghdl_run_flags => [],
         # IV
 	iv => 0,
 	iv_flags => [split(/\s+/,"+define+iverilog -o $self->{obj_dir}/simiv")],
 	iv_flags2 => [],  # Overridden in some sim files
-	ivrun_flags => [],
+	iv_run_flags => [],
 	# VCS
 	vcs => 0,
 	vcs_flags => [split(/\s+/,"+vcs+lic+wait +cli -I +define+VCS+1 -q -sverilog -CFLAGS '-DVCS' ")],
 	vcs_flags2 => [],  # Overridden in some sim files
-	vcsrun_flags => [split(/\s+/,"+vcs+lic_wait")],
+	vcs_run_flags => [split(/\s+/,"+vcs+lic_wait")],
 	# NC
 	nc => 0,
 	nc_flags => [split(/\s+/,"+licqueue +nowarn+LIBNOU +define+NC=1 -q +assert +sv -c ")],
 	nc_flags2 => [],  # Overridden in some sim files
-	ncrun_flags => [split(/\s+/,"+licqueue -q +assert +sv -R")],
+	nc_run_flags => [split(/\s+/,"+licqueue -q +assert +sv -R")],
 	# Verilator
 	vlt => 0,
 	'v3' => 0,
@@ -334,11 +364,6 @@ sub new {
 	%$self};
     bless $self, $class;
 
-    $self->{mode} ||= "atsim" if $self->{atsim};
-    $self->{mode} ||= "vcs" if $self->{vcs};
-    $self->{mode} ||= "vlt" if $self->{vlt};
-    $self->{mode} ||= "nc" if $self->{nc};
-    $self->{mode} ||= "iv" if $self->{iv};
     $self->{VM_PREFIX} ||= "V".$self->{name};
     $self->{stats} ||= "$self->{obj_dir}/V".$self->{name}."__stats.txt";
     $self->{status_filename} ||= "$self->{obj_dir}/V".$self->{name}.".status";
@@ -346,7 +371,13 @@ sub new {
     $self->{coverage_filename} ||= "$self->{obj_dir}/vlt_coverage.pl";
     $self->{vcd_filename}  ||= "$self->{obj_dir}/sim.vcd";
     $self->{main_filename} ||= "$self->{obj_dir}/$self->{VM_PREFIX}__main.cpp";
-    ($self->{top_filename} = $self->{pl_filename}) =~ s/\.pl$/\.v/;
+    ($self->{top_filename} = $self->{pl_filename}) =~ s/\.pl$//;
+    if (-e ($self->{top_filename}.".vhd")) { # If VHDL file exists
+	$self->{vhdl} = 1;
+        $self->{top_filename} .= ".vhd";
+    } else {
+        $self->{top_filename} .= ".v";
+    }
     if (!$self->{make_top_shell}) {
 	$self->{top_shell_filename} = $self->{top_filename};
     } else {
@@ -392,7 +423,6 @@ sub read {
     (-r $self->{pl_filename})
 	or return $self->error("Can't open $self->{pl_filename}\n");
     $Self = $self;
-    $Self = $self;
     delete $INC{$self->{pl_filename}};
     require $self->{pl_filename};
 }
@@ -476,6 +506,14 @@ sub compile {
 
     compile_vlt_flags(%param);
 
+    if (!$self->{make_top_shell}) {
+	$param{top_shell_filename}
+	= $self->{top_shell_filename} = $self->{top_filename};
+    } else {
+	$param{top_shell_filename}
+	= $self->{top_shell_filename} = "$self->{obj_dir}/$self->{VM_PREFIX}__top.".$self->v_suffix;
+    }
+
     if ($param{atsim}) {
 	$self->_make_top();
 	$self->_run(logfile=>"$self->{obj_dir}/atsim_compile.log",
@@ -490,7 +528,25 @@ sub compile {
 			  @{$param{v_other_filenames}},
 			  ]);
     }
-    if ($param{vcs}) {
+    elsif ($param{ghdl}) {
+	mkdir $self->{ghdl_work_dir};
+	$self->_make_top();
+	$self->_run(logfile=>"$self->{obj_dir}/ghdl_compile.log",
+		    fails=>$param{fails},
+		    cmd=>[($ENV{VERILATOR_GHDL}||"ghdl"),
+			  # Add -c here, as having -c twice freaks it out
+			  ((($ENV{VERILATOR_GHDL}||' ') =~ / -c\b/) ? "" : "-c"),
+			  @{$param{ghdl_flags}},
+			  @{$param{ghdl_flags2}},
+			  #@{$param{v_flags}},  # Not supported
+			  #@{$param{v_flags2}}, # Not supported
+			  $param{top_filename},
+			  $param{top_shell_filename},
+			  @{$param{v_other_filenames}},
+			  "-e t",
+			  ]);
+    }
+    elsif ($param{vcs}) {
 	$self->_make_top();
 	$self->_run(logfile=>"$self->{obj_dir}/vcs_compile.log",
 		    fails=>$param{fails},
@@ -504,8 +560,14 @@ sub compile {
 			  @{$param{v_other_filenames}},
 			  ]);
     }
-    if ($param{nc}) {
+    elsif ($param{nc}) {
 	$self->_make_top();
+	my @more_args;
+	if ($self->vhdl) {
+	    ((my $ts = $param{top_shell_filename}) =~ s!\.v!!);
+	    $ts =~ s!.*/!!;;
+	    push @more_args, "-vhdltop", $ts;
+	}
 	$self->_run(logfile=>"$self->{obj_dir}/nc_compile.log",
 		    fails=>$param{fails},
 		    cmd=>[($ENV{VERILATOR_NCVERILOG}||"ncverilog"),
@@ -516,9 +578,10 @@ sub compile {
 			  $param{top_filename},
 			  $param{top_shell_filename},
 			  @{$param{v_other_filenames}},
+			  @more_args
 			  ]);
     }
-    if ($param{iv}) {
+    elsif ($param{iv}) {
 	$self->_make_top();
 	my @cmd = (($ENV{VERILATOR_IVERILOG}||"iverilog"),
 		   @{$param{iv_flags}},
@@ -534,10 +597,10 @@ sub compile {
 		    fails=>$param{fails},
 		    cmd=>\@cmd);
     }
-    if ($param{vlt}) {
+    elsif ($param{vlt}) {
 	my @cmdargs = $self->compile_vlt_flags(%param);
 
-	if ($self->sc_or_sp && !defined $ENV{SYSTEMC}) {
+	if ($self->sc_or_sp && !defined $ENV{SYSTEMC} && !defined $ENV{SYSTEMC_INCLUDE}) {
 	    $self->skip("Test requires SystemC; ignore error since not installed\n");
 	    return 1;
 	}
@@ -575,6 +638,9 @@ sub compile {
 			      ]);
 	}
     }
+    else {
+	$self->error("No compile step for this simulator");
+    }
     return 1;
 }
 
@@ -582,6 +648,7 @@ sub execute {
     my $self = (ref $_[0]? shift : $Self);
     return 1 if $self->errors || $self->skips;
     my %param = (%{$self}, @_);	   # Default arguments are from $self
+    #   params may be expect or {tool}_expect
     $self->oprint("Run\n");
 
     my $run_env = $param{run_env};
@@ -591,39 +658,59 @@ sub execute {
 	$self->_run(logfile=>"$self->{obj_dir}/atsim_sim.log",
 		    fails=>$param{fails},
 		    cmd=>["echo q | ".$run_env."$self->{obj_dir}/athdl_sv",
-			  @{$param{atsimrun_flags}},
+			  @{$param{atsim_run_flags}},
 			  @{$param{all_run_flags}},
-			  ]);
+		          ],
+		    %param,
+		    expect=>$param{atsim_run_expect},	# non-verilator expect isn't the same
+		    );
     }
-    if ($param{iv}) {
+    elsif ($param{ghdl}) {
+	$self->_run(logfile=>"$self->{obj_dir}/ghdl_sim.log",
+		    fails=>$param{fails},
+		    cmd=>[$run_env."$self->{obj_dir}/simghdl",
+			  @{$param{ghdl_run_flags}},
+			  @{$param{all_run_flags}},
+		          ],
+		    %param,
+		    expect=>$param{ghdl_run_expect},	# non-verilator expect isn't the same
+		    );
+    }
+    elsif ($param{iv}) {
 	$self->_run(logfile=>"$self->{obj_dir}/iv_sim.log",
 		    fails=>$param{fails},
 		    cmd=>[$run_env."$self->{obj_dir}/simiv",
-			  @{$param{ivrun_flags}},
+			  @{$param{iv_run_flags}},
 			  @{$param{all_run_flags}},
-			  ]);
+		          ],
+		    %param,
+		    expect=>$param{iv_run_expect},	# non-verilator expect isn't the same
+		    );
     }
-    if ($param{nc}) {
+    elsif ($param{nc}) {
 	$self->_run(logfile=>"$self->{obj_dir}/nc_sim.log",
 		    fails=>$param{fails},
 		    cmd=>["echo q | ".$run_env.($ENV{VERILATOR_NCVERILOG}||"ncverilog"),
-			  @{$param{ncrun_flags}},
+			  @{$param{nc_run_flags}},
 			  @{$param{all_run_flags}},
-			  ]);
+		          ],
+		    %param,
+		    expect=>$param{nc_run_expect},	# non-verilator expect isn't the same
+		    );
     }
-    if ($param{vcs}) {
+    elsif ($param{vcs}) {
 	#my $fh = IO::File->new(">simv.key") or die "%Error: $! simv.key,";
 	#$fh->print("quit\n"); $fh->close;
 	$self->_run(logfile=>"$self->{obj_dir}/vcs_sim.log",
 		    cmd=>["echo q | ".$run_env."./simv",
-			  @{$param{vcsrun_flags}},
+			  @{$param{vcs_run_flags}},
 			  @{$param{all_run_flags}},
 		          ],
 		    %param,
-		    expect=>undef,	# vcs expect isn't the same
+		    expect=>$param{vcs_run_expect},	# non-verilator expect isn't the same
 		    );
     }
-    if ($param{vlt}
+    elsif ($param{vlt}
 	#&& (!$param{needs_v4} || -r "$ENV{VERILATOR_ROOT}/src/V3Gate.cpp")
 	) {
 	$param{executable} ||= "$self->{obj_dir}/$param{VM_PREFIX}";
@@ -634,8 +721,12 @@ sub execute {
 			  @{$param{all_run_flags}},
 			  ],
 		    %param,
+		    expect=>$param{expect},		# backward compatible name
 		    );
     }
+    else {
+	$self->error("No execute step for this simulator");
+    }
 }
 
 sub inline_checks {
@@ -705,6 +796,21 @@ sub top_filename {
     return $self->{top_filename};
 }
 
+sub vhdl {
+    my $self = (ref $_[0]? shift : $Self);
+    $self->{vhdl} = shift if defined $_[0];
+    if ($self->{vhdl}) {
+	$self->{top_filename} =~ s/\.v$/\.vhdl/;
+    }
+    return $self->{vhdl};
+}
+
+sub v_suffix {
+    my $self = (ref $_[0]? shift : $Self);
+    # Suffix for file type, e.g. .vhdl or .v
+    return $self->{vhdl} ? "vhdl" : "v";
+}
+
 sub sp {
     my $self = (ref $_[0]? shift : $Self);
     return $self->{sp};
@@ -816,7 +922,11 @@ sub _run {
 sub _make_main {
     my $self = shift;
 
-    $self->_read_inputs();
+    if ($self->vhdl) {
+	$self->_read_inputs_vhdl();
+    } else {
+	$self->_read_inputs_v();
+    }
 
     my $filename = $self->{main_filename};
     my $fh = IO::File->new(">$filename") or die "%Error: $! $filename,";
@@ -959,8 +1069,17 @@ sub _print_advance_time {
 
 sub _make_top {
     my $self = shift;
+    if ($self->vhdl) {
+	$self->_make_top_vhdl;
+    } else {
+	$self->_make_top_v;
+    }
+}
 
-    $self->_read_inputs();
+sub _make_top_v {
+    my $self = shift;
+
+    $self->_read_inputs_v();
 
     my $fh = IO::File->new(">$self->{top_shell_filename}") or die "%Error: $! $self->{top_shell_filename},";
     print $fh "module top;\n";
@@ -1007,6 +1126,61 @@ sub _make_top {
     $fh->close();
 }
 
+sub _make_top_vhdl {
+    my $self = shift;
+
+    $self->_read_inputs_vhdl();
+
+    my $fh = IO::File->new(">$self->{top_shell_filename}") or die "%Error: $! $self->{top_shell_filename},";
+    print $fh "library ieee;\n";
+
+    my @ports = sort (keys %{$self->{inputs}});
+
+    print $fh "entity t_ent is\n";
+    if ($#ports >= 0) {
+	print $fh "       port(\n";
+	my $semi = "";
+	foreach my $inp (@ports) {
+	    print $fh "\t${semi}${inp} : in std_logic\n";
+	    $semi=";";
+	}
+	print $fh "    );\n";
+    }
+    print $fh "end;\n";
+
+    print $fh "entity top is\n";
+    print $fh "end;\n";
+
+    # Inst
+    print $fh "architecture t_beh of t_ent is\n";
+    if ($#ports >= 0) {
+	foreach my $inp (@ports) {
+	    print $fh "\tsignal ${inp} : std_logic := '0';\n";
+	}
+    }
+    print $fh "   begin\n";
+
+    print $fh "    t : t_ent\n";
+    if ($#ports >= 0) {
+	print $fh "       port map(\n";
+	my $comma="";
+	foreach my $inp (@ports) {
+	    print $fh "\t${comma}${inp} => ${inp}\n";
+	    $comma=",";
+	}
+	print $fh "    )\n";
+    }
+    print $fh "   ;\n";
+
+    print $fh "   end;\n";
+
+    # Waves TBD
+    # Test TBD
+
+    print $fh "end;\n";
+    $fh->close();
+}
+
 #######################################################################
 
 sub _sp_preproc {
@@ -1025,9 +1199,9 @@ sub _sp_preproc {
 
 #######################################################################
 
-sub _read_inputs {
+sub _read_inputs_v {
     my $self = shift;
-    my $filename = $self->{top_filename};
+    my $filename = $self->top_filename;
     $filename = "$self->{t_dir}/$filename" if !-r $filename;
     my $fh = IO::File->new("<$filename") or die "%Error: $! $filename,";
     while (defined(my $line = $fh->getline)) {
@@ -1041,6 +1215,27 @@ sub _read_inputs {
     $fh->close();
 }
 
+#######################################################################
+
+sub _read_inputs_vhdl {
+    my $self = shift;
+    my $filename = $self->top_filename;
+    $filename = "$self->{t_dir}/$filename" if !-r $filename;
+    my $fh = IO::File->new("<$filename") or die "%Error: $! $filename,";
+    while (defined(my $line = $fh->getline)) {
+	# Only supports this form right now:
+	# signal: in ...
+	# signal: out ...
+	if ($line =~ /^\s*(\S+)\s*:\s*(in)\b\s*/) {
+	    $self->{inputs}{$1} = $1;
+	}
+	if ($line =~ /^\s*(architecture)/) {
+	    last;
+	}
+    }
+    $fh->close();
+}
+
 #######################################################################
 # Verilator utilities
 
@@ -1277,6 +1472,10 @@ Requires --debug.
 
 Run Verilator generated executable under the debugger.
 
+=item --ghdl
+
+Run using GHDL.
+
 =item --golden
 
 Update golden files, equivalent to setting HARNESS_UPDATE_GOLDEN=1.
@@ -1288,7 +1487,7 @@ Displays this message and program version and exits.
 =item --j #
 
 Run number of parallel tests, or 0 to determine the count based on the
-number of cores installed.  Requires Parallel::Forker project.
+number of cores installed.  Requires Perl's Parallel::Forker package.
 
 =item --optimize
 
@@ -1327,7 +1526,15 @@ Run using Verilator.
 
 =item SYSTEMC
 
-Root directory name of SystemC kit.
+Root directory name of SystemC kit.  Only used if SYSTEMC_INCLUDE not set.
+
+=item SYSTEMC_INCLUDE
+
+Directory name with systemc.h in it.
+
+=item VERILATOR_GHDL
+
+Command to use to invoke GHDL.
 
 =item VERILATOR_IVERILOG
 
@@ -1351,7 +1558,7 @@ Command to use to invoke VCS.
 
 The latest version is available from L<http://www.veripool.org/>.
 
-Copyright 2003-2011 by Wilson Snyder.  Verilator is free software; you can
+Copyright 2003-2012 by Wilson Snyder.  Verilator is free software; you can
 redistribute it and/or modify it under the terms of either the GNU Lesser
 General Public License Version 3 or the Perl Artistic License Version 2.0.
 
