@@ -10,7 +10,7 @@ if (!$::Driver) { use FindBin; exec("$FindBin::Bin/bootstrap.pl", @ARGV, $0); di
 $Self->{vlt} or $Self->skip("Verilator only test");
 
 compile (
-	 v_flags2 => ["--debug-sigsegv"],
+	 verilator_flags2 => ["--debug-sigsegv"],
 	 fails=>$Self->{v3},
 	 expect=>
 '%Error: Verilator internal fault, sorry.  Consider trying --debug --gdbbt
