@@ -10,14 +10,14 @@ if (!$::Driver) { use FindBin; exec("$FindBin::Bin/bootstrap.pl", @ARGV, $0); di
 top_filename("t/t_trace_ena.v");
 
 compile (
-	 v_flags2 => [$Self->{v3}?'-trace -sp':''],
+	 verilator_flags2 => ['-trace -sp'],
 	 );
 
 execute (
 	 check_finished=>1,
 	 );
 
-if ($Self->{v3}) {
+if ($Self->{vlt}) {
     # Note more checks in _cc.pl
     file_grep     ("$Self->{obj_dir}/simx.vcd", qr/\$enddefinitions/x);
 }
